' PROGRAM FOR SMOOTH PYCNOPHYLACTIC SPATIAL INTERPOLATION
' MS QuickBasic 45 from original Fortran program
'
' Copyright 1994 Waldo R. Tobler
' Original version 1976; PC version August 1992; revised June 1994
'
' See: W.Tobler, 1979, "Smooth Pycnphylactic Interpolation for
'      Geographical Regions", J.Am.Stat.Assn, 74, 367 (Sept); 519-536
'
' Professor Waldo Tobler
' Department of Geography
' University of California
' Santa Barbara, CA 93106-4060
' tobler@geog.ucsb.edu
'
' Mass preserving spatial interpolation for pointless data.
' For use with aggregate census (count) data.
' Needed is an ID array which identifies each geographic zone,
' and a list of numerical values for each zone.
'
' Program variables
' Nr = No. of Zones.
' Rw,Cl define the row and column size of the array.
' Iter = number of iterations requested.
' Con$="C" to continue prior iterations, "I" otherwise (default)
' Bdy$="D" results in Dirichlet condition, value at edge=zero (default)
' Bdy$="N" for Dz/Dn=0; Neumann condition, normal derivative at edge = 0
' Sm$="L" yields the LaPlacian definition of smoothness (default)
' Sm$="B" yields the biharmonic definition of smoothness
' Ne$="Y" allows negative data (e.g., for net migration). "N" is default
'
  DECLARE SUB EDGE (Ir%, Jc%, Nr%)
  OPTION BASE 1
  DEFSNG A-H, O-Z: DEFINT I-N
  CLS
  Ver$ = " Program version: June 1994."
  PRINT "SMOOTH PYCNOPHYLACTIC SPATIAL INTERPOLATION, BY W.TOBLER"
  PRINT
 'Start screen input
  PRINT "Enter your name and a title for the data   ";
  LINE INPUT ; Titl$
  PRINT
  PRINT
  INPUT "Enter the name of the file containing the ID matrix  "; Id$
  OPEN Id$ FOR INPUT AS #1
  PRINT
  PRINT "Is this an intial run or a continuation of previous iterations?"
  INPUT "Initial or Continuation I/C "; Con$
  Con$ = UCASE$(MID$(Con$, 1, 1))
  IF Con$ = "" THEN Con$ = "I"
  IF Con$ <> "I" THEN PRINT "The area values must now come from an array"
  PRINT
  INPUT "Enter the name of the file containing area values  "; Hh$
  OPEN Hh$ FOR INPUT AS #3
  PRINT
  PRINT "Enter the name of the file to receive printer output"
  INPUT "(default is OUT.DAT)  "; Out$
  IF Out$ = "" THEN Out$ = "out.dat"
  OPEN Out$ FOR OUTPUT AS #2
  PRINT
  PRINT "Enter the name of the file to receive the interpolated array"
  INPUT "(default is ZOUT.DAT)  "; Zout$
  IF Zout$ = "" THEN Zout$ = "zout.dat"
  OPEN Zout$ FOR OUTPUT AS #4
  Fmt$ = "##.######^^^^"
  FMT2$ = "####"
  FMT3$ = "##.###^^^^"
  Five = 5
  PRINT
  INPUT "How many data zones are there  "; Nr
  INPUT "How many rows in the ID array  "; Rw%
  INPUT "How many columns in the ID array  "; Cl%
  INPUT "Do you wish the input array(s) listed  "; IDout$
  IF IDout$ = "" THEN IDout$ = "N"
  IDout$ = UCASE$(MID$(IDout$, 1, 1))
  PRINT
  PRINT "Specify the boundary condition:"
  INPUT "D -> Dirchlet, N-> Neumann condition  "; Bdy$
  IF Bdy$ = "" THEN Bdy$ = "D"
  Bdy$ = UCASE$(MID$(Bdy$, 1, 1))
  PRINT
  PRINT "Specify the smoothness criterion"
  INPUT "L -> Laplacian, B -> Biharmonic criterion  "; Sm$
  IF Sm$ = "" THEN Sm$ = "L"
  Sm$ = UCASE$(MID$(Sm$, 1, 1))
  PRINT
  INPUT "Are any of the zonal data values negative  "; Ne$
  IF Ne$ = "" THEN Ne$ = "N"
  Ne$ = UCASE$(MID$(Ne$, 1, 1))
TryAgain:
  It$ = "Y"
  PRINT
  INPUT "How many iterations of smoothing  "; Iter
  IF Iter > 2000 THEN
  PRINT "That's a lot of iterations. Do you really want this many Y/N"
  INPUT "(Recall that the program has a restart option)  "; It$
  END IF
  IF It$ = "" THEN GOTO TryAgain
  IF It$ = "n" OR It$ = "N" THEN GOTO TryAgain
  PRINT
 'End of most screen input
  PRINT #2, " "; Titl$
  PRINT #2, Nr; "zones."
  PRINT #2, Rw%; "rows and"; Cl%; "columns in the array."
  IF Bdy$ = "D" THEN
  PRINT #2, " Dirchlet boundary condition specified."
  Neu = 0
  ELSE
  Neu = 1
  PRINT #2, " Neumann boundary condition specified."
  END IF
  PRINT #2, Iter;
  IF Sm$ = "L" THEN
    IF Iter = 1 THEN
    PRINT #2, "Laplacian smoothing requested."
    ELSE
    PRINT #2, "Laplacian smoothings requested."
    END IF
  ELSE
    IF Iter = 1 THEN
    PRINT #2, "Biharmonic smoothing requested."
    ELSE
    PRINT #2, "Biharmonic smoothings requested."
    END IF
  END IF
  IF Ne$ = "Y" THEN PRINT #2, " Some negative data values expected."
  PRINT #2, " The resulting array will be sent to  "; UCASE$(Zout$)
  IF Con$ = "C" THEN PRINT #2, " This is a continuation of a previous run."
  DIM Z(Rw%, Cl%), Dz(Rw%, Cl%), Id(Rw%, Cl%)
  Nr1 = Nr + 1
  DIM Area(Nr1), Valu(Nr1), SumRuf(Nr1), Dens(Nr1), Zd(Nr1), Aa(Nr1)
  RC% = Rw% * Cl%
  ValS = 0!
  ValuSum = 0!
 'Starting to read data files
  Begin = TIMER   'Start timing here
  PRINT "Reading data values."
  FOR K = 1 TO Nr
  Valu(K) = 0!
  Area(K) = 0!
  IF Con$ = "I" THEN
 'Normal input of data
  INPUT #3, Valu(K)
  ValuSum = ValuSum + Valu(K)
  END IF
  IF Valu(K) < 0! AND Ne$ = "N" THEN
  PRINT " Negative value encountered on input."
  PRINT #2, " Some original value was negative"
  INPUT " Should it be made positive ? Y/N "; Ne2$
    IF UCASE$(MID$(Ne2$, 1, 1)) = "Y" THEN
    Valu(K) = ABS(Valu(K))
    PRINT #2, " Absolute value used. Area number "; K
    ELSE
    Ne$ = "Y"
    PRINT #2, " Program instruction modified to allow negative values"
    PRINT " Program instruction modified to allow negative values"
    END IF
  END IF
  NEXT K
  K = 0
  Valu(Nr1) = 0!
  IF Con$ = "C" THEN
 ' Continuation data input
  IF IDout$ = "Y" THEN PRINT #2, " Initial values, upon restarting:"
  PRINT "Reading data array."
  FOR I = 1 TO Rw%
    FOR J = 1 TO Cl%
    INPUT #3, Z(I, J)
    IF Z(I, J) < 0 AND Ne$ = "N" THEN Ne$ = "Y"
    ValuSum = ValuSum + INT(Z(I, J) + .5)
    NEXT J
  IF IDout$ = "Y" THEN
    FOR Kk = 1 TO Cl%
    IF Kk < Cl% THEN
    PRINT #2, USING FMT3$; Z(I, Kk);
    ELSE
    PRINT #2, USING FMT3$; Z(I, Kk)
    END IF
    NEXT Kk
  END IF
  NEXT I
  END IF
 'Read ID table
  PRINT #2,
  IF IDout$ = "Y" THEN PRINT #2, " The original ID array:"
  PRINT "Reading the ID array."
  FOR I = 1 TO Rw%
    FOR J = 1 TO Cl%
    INPUT #1, Id(I, J)
    IF Id(I, J) <= 0 THEN Id(I, J) = Nr1
    M = Id(I, J)
    IF Con$ = "C" THEN Valu(M) = Valu(M) + INT(Z(I, J) + .5)
    Area(M) = Area(M) + 1!
    NEXT J
  IF IDout$ = "Y" THEN
    FOR Kk = 1 TO Cl%
    IDx = Id(I, Kk)
    IF IDx >= Nr1 THEN IDx = 0
      IF Kk < Cl% THEN
      PRINT #2, USING FMT2$; IDx;
      ELSE
      PRINT #2, USING FMT2$; IDx
      END IF
    NEXT Kk
  END IF
  NEXT I
 'All data are in now. Start simple computations
  PRINT "Starting to compute."
  PRINT #2,
  PRINT #2, " Initial values:"
  PRINT #2, "  ZONE NO.    VALUE        AREA          DENSITY"
  SumAreas = 0
  ValMin = 1E+15: Amin = ValMin: Dmin = ValMin
  FOR M = 1 TO Nr1
 'Valu is the count to distribute
 'Area is the area,
 'Aa is the inverse area
 'Dens is the conputed density (Valu*Aa)
  IF (Area(M) = 0!) THEN Aa(M) = 0!
  IF Area(M) <> 0! THEN Aa(M) = 1 / Area(M)
  Dens(M) = Valu(M) * Aa(M)
  SumAreas = SumAreas + Area(M)
  IF (Area(M) <> 0!) THEN
   IF M <> Nr1 THEN
   PRINT #2, M, Valu(M), Area(M), Dens(M)
   SumV = SumV + Valu(M)
   Suma = Suma + Area(M)
   SumD = SumD + Dens(M)
   IF Valu(M) > ValMax THEN ValMax = Valu(M)
   IF Valu(M) < ValMin THEN ValMin = Valu(M)
   IF Area(M) > Amax THEN Amax = Area(M)
   IF Area(M) < Amin THEN Amin = Area(M)
   IF Dens(M) > Dmax THEN Dmax = Dens(M)
   IF Dens(M) < Dmin THEN Dmin = Dens(M)
   ELSE
   PRINT #2, M, Valu(M), Area(M), Dens(M); "(area outside of domain)"
   END IF
  ELSE
   IF M < Nr1 THEN
   PRINT #2, M, Valu(M), Area(M); "            Zero area!"
   SumV = SumV + Valu(M)
   Suma = Suma + Area(M)
   SumD = SumD + Dens(M)
   IF Valu(M) > ValMax THEN ValMax = Valu(M)
   IF Valu(M) < ValMin THEN ValMin = Valu(M)
   IF Area(M) > Amax THEN Amax = Area(M)
   IF Area(M) < Amin THEN Amin = Area(M)
   IF Dens(M) > Dmax THEN Dmax = Dens(M)
   IF Dens(M) < Dmin THEN Dmin = Dens(M)
   ELSE
   PRINT #2, M, Valu(M), Area(M), Dens(M); "(area outside of domain)"
   END IF
  END IF
  SumRuf(M) = 0
  NEXT M
  SumAreas = SumAreas - Area(Nr1)
  Vave = SumV / Nr
  Aave = Suma / Nr
  Dave = SumD / Nr
  PRINT #2,
  PRINT #2, " Maximum      "; ValMax, Amax, Dmax
  PRINT #2, " Average      "; Vave, Aave, Dave
  PRINT #2, " Minimum      "; ValMin, Amin, Dmin
  SumRuf(Nr1) = 0!
  Zd(Nr1) = 0!
  PRINT #2,
  PRINT #2, " Sum of data values:"; ValuSum
  PRINT #2, " Total data area is"; SumAreas; "units out of a total lattice of"; RC%; "units."
 'Set initial densities, unless read in
  IF Con$ = "C" THEN GOTO HaveDens
  FOR J = 1 TO Cl%
   FOR I = 1 TO Rw%
   Ir = Id(I, J)
   Z(I, J) = Dens(Ir)
   NEXT I
  NEXT J
HaveDens:
  IF Iter <= 0 THEN GOTO DunIters
  Cm1% = Cl% - 1
  Rm1% = Rw% - 1
  Cm3% = Cl% - 3
  Rm3% = Rw% - 3
  PRINT "Starting iterations.";
  Check% = INT(Rw% * Cl% / 1000)
  IF Check% < 2 OR Iter < 12 THEN
  Check% = 1
  PRINT " Each dot represents one iteration"
  ELSE
  PRINT " Each dot represents"; Check%; "iterations"
  END IF
  ItCount = 0
  PRINT #2,
  PRINT #2, " Iteration & Average departure from smoothness"
  T3 = 2500!
  T2 = 0!

' START ITERATIONS

  FOR K = 1 TO Iter
  IF K > 1 THEN T2 = SumSqRuf
  SumSqRuf = 0!
  ItCount = ItCount + 1
    IF ItCount >= Check% THEN
    PRINT ".";
    ItCount = 0
    END IF
  IF Bdy$ = "N" THEN CALL EDGE(Rw%, Cl%, Nr)

' FIRST PASS
  FOR J = 1 TO Cl%
  Jm1 = J - 1
  IF Jm1 < 1 THEN Jm1 = 1
  Jp1 = J + 1
  IF Jp1 > Cl% THEN Jp1 = Cl%
    FOR I = 1 TO Rw%
    Ir = Id(I, J)
   'Look at departure from smoothness
    Im1 = I - 1
    IF Im1 < 1 THEN Im1 = 1
    Ip1 = I + 1
    IF Ip1 > Rw% THEN Ip1 = Rw%
    IF Ir > Nr THEN GOTO OutSide
    IF Sm$ = "L" THEN GOTO LaPlace
    IF (J < 3) OR (J > Cm3%) THEN GOTO LaPlace
    IF (I < 3) OR (I > Rm3%) THEN GOTO LaPlace
   'Biharmonic definition of smoothness
    Ave = (Z(Im1, J) + Z(I, Jm1) + Z(Ip1, J) + Z(I, Jp1)) * 8!
    Ave = Ave - (Z(Im1, Jm1) + Z(Im1, Jp1) + Z(Ip1, Jp1) + Z(Ip1, Jm1)) * 2!
    Ave = Ave - (Z(I - 2, J) + Z(I, J - 2) + Z(I, J + 2) + Z(I + 2, J))
    Ave = .05 * Ave
    GOTO GotSmooth
LaPlace:
   'LaPlacian definition of smoothness
    Ave = .25 * (Z(Im1, J) + Z(Ip1, J) + Z(I, Jm1) + Z(I, Jp1))
GotSmooth:
    Roughness = Ave - Z(I, J)
    SumSqRuf = SumSqRuf + Roughness ^ 2
    Roughness = Roughness * .25
    SumRuf(Ir) = SumRuf(Ir) + Roughness
    Dz(I, J) = Roughness
    GOTO EdgeOK
OutSide:
   'This treats parts outside of domain of interest
   'Dirichlet condition, at the edge of the domain
    IF Bdy$ = "D" THEN GOTO EdgeOK
   'Neumann condition on the boundary
    IF (J < 3) OR (J > Cm3%) THEN GOTO NearEdge
    IF (I < 3) OR (I > Rm3%) THEN GOTO NearEdge
    Ave = .25 * (Z(I + 2, J) + Z(I - 2, J) + Z(I, J + 2) + Z(I, J - 2))
    GOTO EdgeOK
NearEdge:
    Ave = .25 * (Z(Im1, J) + Z(Ip1, J) + Z(I, Jp1) + Z(I, Jm1))
EdgeOK:
    IF (Ne$ = "N") AND (Ave < 0!) THEN Ave = 0!
    Z(I, J) = Ave
    NEXT I
  NEXT J
    FOR Ir = 1 TO Nr
   'Decrement by average increment
    SumRuf(Ir) = -SumRuf(Ir) * Aa(Ir)
    Zd(Ir) = 0!
    NEXT Ir

' SECOND PASS
' Apply nomalized adjustment
  FOR J = 1 TO Cl%
   FOR I = 1 TO Rw%
   Ir = Id(I, J)
   IF Ir > Nr THEN GOTO OfNoConcern
   Zt = Z(I, J) + Dz(I, J) + SumRuf(Ir)
   IF (Ne$ = "N") AND (Zt < 0!) THEN Zt = 0!
   Zd(Ir) = Zd(Ir) + Zt
   Z(I, J) = Zt
OfNoConcern:
   NEXT I
  NEXT J
    FOR Ir = 1 TO Nr
    SumRuf(Ir) = 0!
    Zd(Ir) = Valu(Ir) - Zd(Ir)
    Zd(Ir) = Zd(Ir) * Aa(Ir)
    Dens(Ir) = 0!
    Area(Ir) = 0!
    NEXT Ir

' THIRD PASS, for renormalization
' which enforces pycnophylactic condition
  FOR J = 1 TO Cl%
    FOR I = 1 TO Rw%
    Ir = Id(I, J)
    IF Ir > Nr THEN GOTO Exterior
    Area(Ir) = Area(Ir) + 1!
    IF Zd(Ir) > 0! THEN Z(I, J) = Z(I, J) + Zd(Ir)
    IF Zd(Ir) >= 0! THEN GOTO Density
   'Careful treatment to avoid negative values
    IF Ne$ = "Y" THEN Z(I, J) = Z(I, J) + Zd(Ir)
    IF Ne$ = "Y" THEN GOTO Density
    Roughness = -Z(I, J)
    IF Zd(Ir) > Roughness THEN GOTO VeryGood
    Z(I, J) = 0!
    IF Aa(Ir) <= 0! THEN GOTO Density
    A2 = (1! / Aa(Ir)) - Area(Ir)
    IF A2 <= 0! THEN GOTO Density
    Zd(Ir) = Zd(Ir) + (Zd(Ir) - Roughness) / A2
    GOTO Density
VeryGood:
    Z(I, J) = Z(I, J) + Zd(Ir)
Density:
    Dens(Ir) = Dens(Ir) + Z(I, J)
    SumRuf(Ir) = Dens(Ir) - Valu(Ir)
Exterior:
    NEXT I
  NEXT J

 'Stopping rules
  SumSqRuf = SumSqRuf / SumAreas
  IF K < 10 THEN
  PRINT #2, K, SumSqRuf
  ELSEIF INT(K / 10) * 10 = K THEN
  PRINT #2, K, SumSqRuf
  END IF
  IF K = 1 THEN TEST = SumSqRuf
  IF TEST <> 0! THEN T1 = SumSqRuf / TEST
  IF T1 < .00001 THEN GOTO DunIters
  IF T2 <> 0! THEN T3 = SumSqRuf / T2
  IF T3 < .01 THEN GOTO DunIters
  NEXT K
'
' END OF ITERATIONS
'
DunIters:
  IF Iter <> 0 THEN PRINT #2, K, SumSqRuf
  ZnMax = -1E+10
  ZnMin = -ZnMax
  FOR J = 1 TO Cl%
    FOR I = 1 TO Rw%
    Zz = Z(I, J)
    Ir = Id(I, J)
    IF Ir > Nr THEN Z(I, J) = 0!
      IF Zz > ZnMax THEN
        IF Ir < Nr1 THEN
        ZnMax = Zz
        JX = Ir
        END IF
      END IF
      IF Zz < ZnMin THEN
        IF Ir < Nr1 THEN
        ZnMin = Zz
        JN = Ir
        END IF
      END IF
    NEXT I
  NEXT J
  PRINT #2,
  PRINT #2, " Max Density ="; ZnMax; "in area"; JX; "  Min Density ="; ZnMin; "in area"; JN
  IF K = Iter + 1 THEN K = Iter
  Iter = K
  PRINT #2, " Rows ="; Rw%; ", Columns ="; Cl%; ", Zones ="; Nr; ", Iterations ="; K
  PRINT #2,
  PRINT #2, " Values obtained by summation over the resulting array."
  PRINT #2, " Should equal the original input."
  PRINT #2, " Zone        Value"
  ValS = 0!
  FOR K = 1 TO Nr1
  ValS = ValS + Valu(K)
  PRINT #2, K, Valu(K)
  NEXT K
  PRINT #2, " The total is "; ValS
  PRINT #2,
  FOR I = 1 TO Rw%
  ColCount = 0
    FOR Kk = 1 TO Cl%
    IF Kk < Cl% THEN
    ColCount = ColCount + 1
    IF ColCount < Five THEN
    PRINT #4, USING Fmt$; Z(I, Kk);
    PRINT #4, " ";
    ELSE
    PRINT #4, USING Fmt$; Z(I, Kk)
    ColCount = 0
    END IF
    ELSE
    PRINT #4, USING Fmt$; Z(I, Kk)
    END IF
    NEXT Kk
  NEXT I
  PRINT #2, " Resulting interpolated array sent to  "; UCASE$(Zout$)
  PRINT #2, " The ID array came from  "; UCASE$(Id$)
  PRINT #2, " The region data came from  "; UCASE$(Hh$)
  PRINT #2, " This is file  "; UCASE$(Out$)
  PRINT #2, " Title: "; Titl$; "."
  PRINT
  PRINT : PRINT "The program has completed its"; Iter; "iterations."
  PRINT "Look at the file "; UCASE$(Out$); " to see the results"
  PRINT "And use the values in "; UCASE$(Zout$); " for further work."
  Finish = TIMER
  Elapsed = (Finish - Begin) / 60
  Finish = (INT(Elapsed * 100)) / 100
  IF Finish < 0 THEN Finish = 1440 + Finish
  Now$ = MID$(TIME$, 1, 5)
  PRINT "It took"; Finish; "minutes."
    IF Finish > 60 THEN
    Hours% = Finish / 60
    Finish = ((Finish / 60) - Hours%) * 60
      IF Hours% = 1 THEN
      PRINT #2, Hours%; "Hour"; Finish; "minutes, ending at "; Now$; " on "; DATE$
      ELSE
      PRINT #2, Hours%; "Hours"; Finish; "minutes, ending at "; Now$; " on "; DATE$
      END IF
    ELSE
    PRINT #2, " It took"; Finish; "minutes, ending at "; Now$; " on "; DATE$
    END IF
  PRINT #2, Ver$
  PRINT #4,
  PRINT #4, " Title: "; Titl$; "."
  PRINT #4, Nr; "zones."
  PRINT #4, Rw%; "rows and"; Cl%; "columns in the array."
  IF Bdy$ = "D" THEN
  PRINT #4, " Dirchlet boundary condition specified."
  ELSE
  PRINT #4, " Neumann boundary condition specified."
  END IF
  IF Sm$ = "L" THEN
    PRINT #4, " Laplacian smoothing specified."
  ELSE
    PRINT #4, " Biharmonic smoothing specified."
  END IF
  IF Ne$ = "Y" THEN PRINT #4, " Negative data values ok."
  PRINT #4, Iter; "iterations used."
  PRINT #4, " The ID array came from  "; UCASE$(Id$)
  PRINT #4, " The region data came from  "; UCASE$(Hh$)
  PRINT #4, " This is file  "; UCASE$(Zout$)
  PRINT #4, " Also look at file  "; UCASE$(Out$)
  PRINT #4, " Finished at "; Now$; " on "; DATE$
  CLOSE
  IF Iter > 10 THEN
  BEEP: BEEP: BEEP
  END IF
  END

  SUB EDGE (Ir, Jc, Nr)
' ASSIGNS VALUES AT EDGE
' ONLY USED FOR NEUMANN CONDITION
  SHARED Z(), Id()
  Lc = Jc - 1
  FOR I = 1 TO Ir
 'Left edge
  J = 1
  IF Id(I, J) <= Nr THEN GOTO RightEdge
  A = Z(I, J + 2)
  Z(I, J) = A
RightEdge:
  J = Jc
  IF Id(I, J) <= Nr THEN GOTO RightDone
  A = Z(I, J - 2)
  Z(I, J) = A
RightDone:
  NEXT I
  FOR J = 2 TO Lc
 'Top
  I = 1
  IF Id(I, J) <= Nr THEN GOTO Bottom
  A = Z(I + 2, J)
  Z(I, J) = A
Bottom:
  I = Ir
  IF Id(I, J) <= Nr THEN GOTO BottomDone
  A = Z(I - 2, J)
  Z(I, J) = A
BottomDone:
  NEXT J
 'Top left
  IF Id(1, 1) <= Nr THEN GOTO TopRight
  A = Z(3, 1)
  Z(1, 1) = .5 * (Z(1, 1) + A)
TopRight:
  IF Id(1, Jc) <= Nr THEN GOTO BotLeft
  A = Z(3, Jc)
  Z(1, Jc) = .5 * (Z(1, Jc) + A)
BotLeft:
  IF Id(Ir, 1) <= Nr THEN GOTO BotRight
  A = Z(Ir - 2, 1)
  Z(Ir, 1) = .5 * (Z(Ir, 1) + A)
BotRight:
  IF Id(Ir, Jc) <= Nr THEN GOTO BotDone
  A = Z(Ir - 2, Jc)
  Z(Ir, Jc) = .5 * (Z(Ir, Jc) + A)
BotDone:
   EXIT SUB
   END SUB

