' To calculate City Densities, for plotting
' Based on the notion that City Area = f(Population)
' and that city radius = sqr ( City Area / Pi)
' with f(Population) = A * P ^B where
' A, B, and Pi are defined below
' W. Tobler, 2/27/92, 5/06/96
' Modification needed: Compute & display density at center.
DECLARE SUB Quikview (Rows%, Cols%, Zmin!, Zmax!, Pop!)
DECLARE SUB Profile (Rows%, Cols%, Xmin!, Xmax!, Ymin!, Ymax!, Zmin!, Zmax!, Pop!)
DECLARE FUNCTION Max! (a!, B!)
DECLARE FUNCTION Min! (a!, B!)
DECLARE FUNCTION TimeDif$ (T1$, T2$)
DECLARE FUNCTION Xint! (X1!, X2!, F1!, F2!, Fi!)
DECLARE SUB SetUpTri (Rows%, Cols%, Dz!)
DECLARE SUB Contour (Rows%, Cols%, Zmin!, Zmax!)
DECLARE SUB PlotSize (Xmin!, Xmax!, Ymin!, Ymax!)
DECLARE SUB Order (X1!, X2!, X3!, Y1!, Y2!, Y3!, F1!, F2!, F3!)
DECLARE SUB DrawGrid (Rows%, Cols%, Xmax!, Xmin!, Ymax!, Ymin!)
DECLARE SUB TriCon (X1!, X2!, X3!, Y1!, Y2!, Y3!, F1!, F2!, F3!, H!)
DECLARE SUB IsoPlot (Rows%, Cols%, Xmin!, Xmax!, Ymin!, Ymax!, Zmin!, Zmax!, C!)
COMMON Z(), Scrn, Titl$
DEFINT I-N
OPTION BASE 1
CLS
CLOSE
' Constants for western cities
' A = .0352    for values in km
' A = .0219    for values in miles
B = .44
Pi = 3.14159265#
Tpi = 2 * Pi
Fmt$ = "######.#"
K = 0: Zmax = -1
COLOR 14
PRINT "            Program to display population densities for cities"
COLOR 15
PRINT
INPUT "   Do you want results in kilometers or miles (k/m) "; Mk$
IF Mk$ = "" THEN Mk$ = "K"
Mk$ = UCASE$(Mk$)
  IF Mk$ = "M" THEN
    a = .0219
  ELSE
    a = .0352
  END IF
INPUT "   Do you wish to have an estimate of the city area and 'radius' (Y/N) "; Size$
IF Size$ = "" THEN Size$ = "Y"
Size$ = UCASE$(Size$)
Redo1:
IF Size$ = "Y" THEN
  INPUT "   Enter city population ", Pop
  If Pop <= 0 then
    Color 12
    Print Pop;"is not a valid population size. Please";
    Color 15
    goto Redo1
  End If  
END IF
Radius = a * Pop ^ B
CityPop = Pop
CityRad = INT(Radius) + CINT(1000 * (Radius - INT(Radius)) + .005) / 1000
CityArea = Pi * Radius * Radius
AreaCity = INT(CityArea) + CINT(100 * (CityArea - INT(CityArea)) + .05) / 100
Circum = Tpi * Radius
Circum = INT(Circum) + CINT(100 * (Circum - INT(Circum)) + .05) / 100
IF Size$ = "Y" THEN
  COLOR 14
  PRINT
  PRINT "   A city with a population of"; Pop; "is likely"
  PRINT "   to have an area of"; AreaCity;
  IF Mk$ = "K" THEN
    PRINT "square kilometers,"
    PRINT "   which is equivalent to a circle of radius"; CityRad; "kilometers"
    PRINT "   with a circumference of"; Circum; "kilometers."
  ELSE
    PRINT "square miles,"
    PRINT "   which is equivalent to a circle of radius"; CityRad; "miles"
    PRINT "   with a circumference of"; Circum; "miles."
  END IF
  COLOR 15
END IF
print 
TryAgain:
PRINT "   Select a city model "
PRINT
PRINT "    1) Cylindric"
PRINT "    2) Conical"
PRINT "    3) Parabolic"
PRINT "    4) Cosine"
PRINT "    5) Exponential"
PRINT "    6) Gaussian"
PRINT
INPUT "   Enter a number to select a city type    ", City
IF City < 0 OR City > 6 THEN 
  Color 12
  print City;"is not a valid option. Please";
  color 15   
  GOTO TryAgain
end If
PRINT
'The following are strictly for the display
Pop = 100000
a = .0352 ' this makes the diagrams look better
Radius = a * Pop ^ B
Size% = 61
Buf = Radius * 1.025
Radius2 = Radius * Radius
PiSqr = Pi * Pi
PiOvR = Pi / Radius
Area = Pi * Radius2
PopOvArea = Pop / Area
PoA3 = 3 * PopOvArea
PoA6 = 6 * PopOvArea
'compute values for grid
Center% = (Size% - 1) / 2 + 1
Dx = 2 * Buf / (Size% - 1)
Nr = Size%
Nc = Size%
NrP1 = Nr + 1
NcP1 = Nc + 1
'PRINT "   Now computing. Please wait."
'Just compute the upper right quadrant
Now1$ = MID$(TIME$, 1, 5)
REDIM Z(Size%, Size%)
PRINT "   ";
FOR I = 1 TO Center%
  Y = Buf - (I - 1) * Dx
  FOR J = Center% TO Nc
    X = -Buf + (J - 1) * Dx
  ' get distance from center
    Distance2 = X * X + Y * Y
  ' Compute population density
    IF Distance2 >= Radius2 THEN
      Dens = 0!
      GOTO outside
    END IF
    SELECT CASE City
    CASE 1
    ' Cylindric shape
      Dens = PopOvArea
      CntrDens = CityPop / (Pi * CityArea)
      Titl$ = "A cylindric city"
    CASE 2
    ' Conical shape
      Distance = SQR(Distance2)
      Dens = PoA3 * (1 - (Distance / Radius))
      CntrDens = 3 * CityPop / CityArea
      Titl$ = "A conical city"
    CASE 3
    ' Parabolic shape
      Distance = SQR(Distance2)
      Dens = PoA6 * ((Radius - Distance) ^ 2) / Radius2
      CntrDens = 6 * CityPop / CityArea
      Titl$ = "A parabolic city"
    CASE 4
    ' Cosine shape
      Distance = SQR(Distance2)
      Dens = (1 + COS(PiOvR * Distance)) * (Pi * Pop / (Radius2 * (PiSqr - 4)))
      CntrDens = Tpi * CityPop / (CityRad * CityRad * (Pi * Pi - 4))
      Titl$ = "A cosine city"
    CASE 5
    ' Exponential decay shape
      Distance = SQR(Distance2)
      Dens = Tpi * EXP(-Distance)
      CntrDens = Tpi
      Titl$ = "An exponential city"
    CASE 6
    ' Gaussian shape
      Dens = Tpi * EXP(-(Distance2) / 5)
      CntrDens = Tpi / 5
      Titl$ = "A Gaussian city"
    END SELECT
outside:
    Z(I, J) = Dens
   'Now fill in symmetrical cells
    Z(NrP1 - I, J) = Dens
    Z(NrP1 - I, NcP1 - J) = Dens
    Z(I, NcP1 - J) = Dens
    K = K + 1
    Zmax = Max!(Zmax, Z(I, J))
  NEXT J
  PRINT ".";
NEXT I
PRINT
IF City = 1 THEN Zmax = 3 * Zmax
'PRINT "Zmax="; Zmax
'STOP
Radius = INT(Radius) + CINT(1000 * (Radius - INT(Radius)) + .005) / 1000
CALL Quikview(Nr, Nc, Zmin, Zmax, Pop)
PRINT
COLOR 14
PRINT "   D O N E"
COLOR 15
PRINT
Schluss:
Now2$ = MID$(TIME$, 1, 5)
IF TimeDif$(Now1$, Now2$) <> "Short" THEN
  PRINT "   Started at  "; Now1$; " on "; DATE$
  PRINT "   Finished at "; Now2$; " on "; DATE$
  PRINT
END IF
COLOR 14
PRINT "   "; Titl$;
PRINT " was chosen and, for a city of"; CityPop; "persons, this yields"
PRINT "   a central density of"; CntrDens; "persons / square unit."
PRINT
PRINT
PRINT
PRINT
PRINT "   Thank Waldo Tobler For this Program"
PRINT
PRINT
PRINT
COLOR 3
PRINT "   Please press 'enter' to end."
DO WHILE INKEY$ = "": LOOP
END

DEFINT I-N
SUB DrawGrid (Rows%, Cols%, Xmax, Xmin, Ymax, Ymin)
'Draw the grid
SHARED Scrn
Root% = INT(SQR(Rows% * Cols%))
IF Root% >= 51 THEN
  Delta% = 10
  GOTO GotGrid
END IF
FOR K = 10 TO 51 STEP 10
  IF Root% < K THEN
    Delta% = K / 10
    GOTO GotGrid
  END IF
NEXT K
GotGrid:
IF Scrn <> 11 THEN COLOR 3
'Draw vertical grid
FOR I = 1 TO Cols% STEP Delta%
  PSET (I, Ymin + Delta% / 2)
  LINE -(I, Ymin - Delta% / 2), , , &H1111
  IF I = 1 OR I = Cols% THEN
    IF Scrn <> 11 THEN COLOR 10
    PSET (I, Ymin)
    LINE -(I, Ymax)
  ELSE
    IF Scrn <> 11 THEN COLOR 3
    PSET (I, Ymin)
    LINE -(I, Ymax)
  END IF
  IF Scrn <> 11 THEN COLOR 3
  PSET (I, Ymax)
  LINE -(I, Ymax - Delta% / 2), , , &H1111
NEXT I
'Draw horizontal grid
FOR I = Rows% TO 1 STEP -Delta%
  PSET (Xmin + Delta% / 2, I)
  LINE -(Xmin, I), , , &H1111
  IF I = 1 OR I = Rows% THEN
    IF Scrn <> 11 THEN COLOR 10
    PSET (Xmin, I)
    LINE -(Xmax, I)
  ELSE
    IF Scrn <> 11 THEN COLOR 3
    PSET (Xmin, I)
    LINE -(Xmax, I)
  END IF
  IF Scrn <> 11 THEN COLOR 3
  PSET (Xmax, I)
  LINE -(Xmax - Delta% / 2, I), , , &H1111
NEXT I
IF Scrn <> 11 THEN COLOR 3
'PRINT "The Grid interval is"; Delta%;
'COLOR 12
'PRINT "Please wait. Computing contours."
'IF Scrn <> 11 THEN COLOR 3
EXIT SUB
END SUB

DEFSNG A-H, O-Z: DEFINT I-N
'Isometric diagram plotting subprogram.
'Conversion by W. Tobler, Dec 1993, to a BASIC subroutine from a FORTRAN
'translation of a 1965 MAD University of Michigan program by W. Tobler.
  SUB IsoPlot (Rows%, Cols%, Xmin, Xmax, Ymin, Ymax, Zmin, Zmax, Pop)
  SHARED Z(), Scrn, Titl$
 'ALF = Isometric angle in degrees (default = 30.)
  Alf = 30!
  Rad = .0174532925#
  Alf = Alf * Rad
  Salf = SIN(Alf)
  Calf = COS(Alf)
  Rw = Rows%: Cl = Cols%
  Grid% = FIX(Max!(Rw, Cl)) + 1
  REDIM Xl(Grid%), Yl(Grid%)
 'Foreshorten
  Salf = .25 * Salf
  Calf = .25 * Calf
  Front = Xmax - Xmin
  Side = (Ymax - Ymin) * Calf
  Wide = Front + Side
 'Get amount to translate to center
  Dx = Side / 2
 'VERTICAL SCALE
 'Scaling data height
  Zrange = Zmax - Zmin
  Hgt = (Ymax - Ymin) * Salf
  Dz = Hgt / Zrange
 'The next 7 lines enlarge the drawing a bit.
  Xmid = (Xmin + Wide) / 2
  Ymid = (Ymin + 2 * Hgt) / 2
  Xn = Xmid - Wide / 2 - 2 * Dx
  Xx = Xmid + Wide / 2 + Dx
  Yn = Ymid - Hgt
  Yx = Ymid + Hgt + Dx / 2
  WINDOW (Xn, Yn)-(Xx, Yx)
 'This is slightly screen dependent
 'Scaling height to be a fraction of side height
 'Dz = .6 * Dz
  R% = Rows%
  R1% = 1
  Dr% = 1
 'PRINT R%; R1%, Dr%
Rescale:
 'BEGIN PLOTTING
  REDIM Xl(Grid%), Yl(Grid%)
  CLS
  IF Scrn <> 11 THEN COLOR 14
 'GOTO SkipRowProf      'To test program; deactivate when ok
 'PRINT "Row profiles"
 'DRAW PROFILES ACROSS THE ROWS
  FOR I = R% TO R1% STEP -Dr%
    K = R% - I + R1%
   'IF C < 0 THEN            'Switch this to take out hidden lines
      Dy = I - Ymin
    'ELSE
     'Dy = K - Ymin
    'END IF
     Cy = Dy * Calf
     Yc = Dy * Salf + Ymin
     FOR J = 1 TO Cols%
       Yl(J) = Yc + Dz * (Z(K, J) - Zmin)
       Xl(J) = Cy + J
       IF J = 1 THEN
        'PRINT J; Yl(J); Xl(J)
        'INPUT X$
        'IF Yl(1) > 1E+10 ^ 5 THEN Yl(1) = Ymax
        'IF Xl(1) > 1E+10 ^ 5 THEN Xl(1) = Xmax
         PSET (Xl(1) - Dx, Yl(1))
       ELSE
         LINE -(Xl(J) - Dx, Yl(J))
       END IF
     NEXT J
   NEXT I
SkipRowProf:
   'GOTO SkipColProf       'To test program; deactivate when ok
   'PRINT "Column profiles"
   'DRAW PROFILES UP THE COLUMNS
    IF Rows% > 40 AND Cols% > 40 THEN
     'Screen resolution is too poor to show both dense row & column profiles
      ColProf% = Cols% - 1
    ELSE
      ColProf% = 1
    END IF
    FOR J = Cols% TO 1 STEP -ColProf%
      Xc = J
     'PRINT R%, R1%, Dr%
      FOR I = R1% TO R% STEP Dr%
       'PRINT "I,K"; I; K
        K = R% - I + R1%
       'IF C > 0 THEN    'Switch this to take out hidden lines
          Dy = I - Ymin
       'ELSE
         'Dy = K - Ymin
       'END IF
        Yl(I) = Dy * Salf + Ymin + Dz * (Z(K, J) - Zmin)
        Xl(I) = Xc + Calf * Dy
        IF I = R1% THEN
          PSET (Xl(R1%) - Dx, Yl(R1%))
        ELSE
          LINE -(Xl(I) - Dx, Yl(I))
        END IF
      NEXT I
    NEXT J
SkipColProf:
    GOTO skipbot               'To test program; deactivate when ok
   'PUT BOTTOM ON THE DIAGRAM
    REDIM Xl(6), Yl(6)
    Yy = Rows%
    Xl(1) = Calf * (Yy - Ymin) + Xmax - Dx
    Yl(1) = Salf * (Yy - Ymin) + Dz * (Z(R%, Cols%) - Zmin) + Ymin
    Xl(2) = Calf * (Yy - Ymin) + Xmax - Dx
    Yl(2) = Salf * (Yy - Ymin) + Ymin
    Xl(3) = Xmax - Dx
    Yl(3) = Ymin
    Xl(4) = Xmin - Dx
    Yl(4) = Ymin
    Xl(5) = Xmin - Dx
    Yl(5) = Dz * (Z(R1%, 1) - Zmin) + Ymin
    Xl(6) = Xmax - Dx
    Yl(6) = Dz * (Z(R1%, Cols%) - Zmin) + Ymin
    PSET (Xl(1), Yl(1))
    LINE -(Xl(2), Yl(2))
    LINE -(Xl(3), Yl(3))
    LINE -(Xl(4), Yl(4))
    LINE -(Xl(5), Yl(5))
    PSET (Xl(3), Yl(3))
    LINE -(Xl(6), Yl(6))
skipbot:
    IF Scrn <> 11 THEN COLOR 3
    PRINT "  "; Titl$; ". Isometric diagram. Hidden lines not deleted."
    INPUT "  Enter +/- to rescale exaggeration, 'enter' to continue ", X$
    IF X$ = "+" THEN
      Dz = Dz * 1.25
      GOTO Rescale
    END IF
    IF X$ = "-" THEN
      Dz = .5 * Dz
      GOTO Rescale
    END IF
  END SUB

FUNCTION Max! (a, b)
Max! = a
IF b > a THEN Max! = b
EXIT FUNCTION
END FUNCTION

FUNCTION Min! (a, b)
Min! = a
IF b < a THEN Min! = b
EXIT FUNCTION
END FUNCTION

      SUB Order (X1, X2, X3, Y1, Y2, Y3, F1, F2, F3)
      DEFINT I-N
      DIM X(3), Y(3), F(3)
      X(1) = X1
      X(2) = X2
      X(3) = X3
      Y(1) = Y1
      Y(2) = Y2
      Y(3) = Y3
      F(1) = F1
      F(2) = F2
      F(3) = F3
      Fmax = Max!(F1, F2)
      Fmax = Max!(Fmax, F3)
      Fmin = Min!(F1, F2)
      Fmin = Min!(Fmin, F3)
      FOR K = 1 TO 3
        IF F(K) = Fmax THEN K3 = K
        IF F(K) = Fmin THEN K1 = K
      NEXT K
      FOR K = 1 TO 3
        IF K <> K1 AND K <> K3 THEN K2 = K
      NEXT K
      F1 = F(K1)
      F2 = F(K2)
      F3 = F(K3)
      X1 = X(K1)
      X2 = X(K2)
      X3 = X(K3)
      Y1 = Y(K1)
      Y2 = Y(K2)
      Y3 = Y(K3)
      EXIT SUB
    END SUB

SUB PlotSize (Xmin, Xmax, Ymin, Ymax)
SHARED Scrn
'Size the data for a drawing
SCREEN Scrn
Dx = Xmax - Xmin
Dy = Ymax - Ymin
Range = Max!(Dx, Dy)
Xmid = Xmin + Dx / 2
Ymid = Ymin + Dy / 2
Xn = Xmid - Range
Xx = Xmid + Range
Yn = Ymid - .8 * Range
Yx = Ymid + .8 * Range
WINDOW (Xn, Yn)-(Xx, Yx)
EXIT SUB
END SUB

  SUB Profile (Rows%, Cols%, Xmin, Xmax, Ymin, Ymax, Zmin, Zmax, Pop)
' Profile diagram plotting subprogram.
  DEFSNG A-H, O-Z: DEFINT I-N
  SHARED Z(), Scrn, Titl$
 'ALF = Isometric angle in degrees (default = 30.)
  Alf = 30!
  Rad = .0174532925#
  Alf = Alf * Rad
  Salf = SIN(Alf)
  Calf = COS(Alf)
  Rw = Rows%: Cl = Cols%
  Grid% = FIX(Max!(Rw, Cl)) + 1
  REDIM Xl(Grid%), Yl(Grid%)
 'Foreshorten
  Salf = .75 * Salf
  Calf = .75 * Calf
  Front = Xmax - Xmin
  Side = (Ymax - Ymin) * Calf
  Wide = Front + Side
 'Get amount to translate to center
  Dx = Side / 2
 'VERTICAL SCALE
 'Scaling data height
  Zrange = Zmax - Zmin
  Hgt = (Ymax - Ymin) * Salf
  Dz = Hgt / Zrange
 'The next 7 lines enlarge the drawing a bit.
  Xmid = (Xmin + Wide) / 2
  Ymid = (Ymin + 2 * Hgt) / 2
  Xn = Xmid - Wide / 2 - 2 * Dx
  Xx = Xmid + Wide / 2 + Dx
  Yn = Ymid - Hgt
  Yx = Ymid + Hgt + Dx / 2
  WINDOW (Xn, Yn)-(Xx, Yx)
 'Scaling height to be a fraction of side height
 'Dz = .6 * Dz
  R% = Rows% / 2 + .5
RescaleIt:
 'BEGIN PLOTTING
  REDIM Xl(Grid%), Yl(Grid%)
  CLS
  IF Scrn <> 11 THEN COLOR 14
 'DRAW PROFILE THRU THE CITY CENTER
  K = R%
  Dy = R% - Ymin
  Cy = Dy * Calf
  Yc = Dy * Salf + Ymin
  FOR J = 1 TO Cols%
    Yl(J) = Yc + Dz * (Z(K, J) - Zmin)
    Xl(J) = Cy + J
    IF J = 1 THEN
      PSET (Xl(1) - Dx, Yl(1))
    ELSE
      LINE -(Xl(J) - Dx, Yl(J))
    END IF
  NEXT J
 'PUT BOTTOM ON THE DIAGRAM
  PSET (Xn, Yl(1))
  LINE -(Xl(1) - Dx, Yl(1)), , , &H1111
  PSET (Xl(Cols%) - Dx, Yl(Cols%))
  LINE -(Xx, Yl(Cols%)), , , &H1111
  IF Scrn <> 11 THEN COLOR 3
  PRINT "  "; Titl$; ". Profile through the center of the city."
  INPUT "  Enter +/- to rescale exaggeration, 'enter' to quit ", X$
  IF X$ = "+" THEN
    Dz = Dz * 1.25
    GOTO RescaleIt
  END IF
  IF X$ = "-" THEN
    Dz = .5 * Dz
    GOTO RescaleIt
  END IF
  COLOR 15
END SUB

SUB Quikview (Rows%, Cols%, Zmin, Zmax, Pop)
' Program to contour and display results.
' This program is intended for quick views, not final displays.
' It works for arbitrary arrays.
' Waldo Tobler, 1/15/94, revised 5/13/95, 6/7/95
' Name changed to quikview from quiklook
' tobler@geog.ucsb.edu
DEFSNG A-H, O-Z: DEFINT I-N
SHARED Z(), Scrn, Titl$
Scrn = 9
IF Scrn <> 11 THEN COLOR 3
COLOR 15
PRINT "   Now setting up for the displays so that the program"
PRINT "   can give you a quick look at the results. Just a moment"
PRINT
Xmin = 1
Xmax = Cols%
Ymin = 1
Ymax = Rows%
Dz = Zmax - Zmin
INPUT "   Do you want a contour map (Y/N) "; Ci$
Ci$ = UCASE$(Ci$)
IF Ci$ = "N" THEN
  Dz = -1
  GOTO SkipMap
END IF
Dz = Dz / 5
'PRINT "   The contour interval on the map will be"; Dz
'INPUT "   Do you wish to change the contour interval Y/N "; Dci$
'Dci$ = UCASE$(Dci$)
'IF UCASE$(Dci$) <> "N" THEN
 ' PRINT
 ' PRINT "   Making the contour interval larger is usually better"
 ' PRINT
 ' INPUT "   Enter a new contour interval "; Dz
'END IF
  INPUT "   Press enter to see the contour map ", X$
  CALL PlotSize(Xmin, Xmax, Ymin, Ymax)
  CLS
 'Do contouring
  IF Scrn <> 11 THEN COLOR 3
  PRINT Titl$; ". ";
 'Print contour interval
 'PRINT "The contour interval is"; Dz; " The array is"; Rows%; "by"; Cols%;
  IF Scrn <> 11 THEN COLOR 3
  CALL DrawGrid(Rows%, Cols%, Xmin, Xmax, Ymin, Ymax)
  CALL SetUpTri(Rows%, Cols%, Dz)
 'Draw bounding rectangle
  IF Scrn <> 11 THEN COLOR 3
  PSET (Xmin, Ymin)
  LINE -(Xmax, Ymin)
  LINE -(Xmax, Ymax)
  LINE -(Xmin, Ymax)
  LINE -(Xmin, Ymin)
SkipMap:
IF Ci$ <> "N" THEN
  INPUT "Do you wish to see an isometric sketch (Y/N) "; Sktch$
ELSE
  INPUT "   Do you wish to see an isometric sketch (Y/N) "; Sktch$
END IF
IF UCASE$(Sktch$) <> "N" THEN
  IF Ci$ <> "N" THEN
  INPUT "Press enter when ready for the isometric diagram.                           ", X$
ELSE
  INPUT "   Press enter when ready for the isometric diagram.                           ", X$
END IF
CALL PlotSize(Xmin, Xmax, Ymin, Ymax)
IF Scrn <> 11 THEN COLOR 3
'Do isometric plot
 CALL IsoPlot(Rows%, Cols%, Xmin, Xmax, Ymin, Ymax, Zmin, Zmax, Pop)
 INPUT "  Do you want to see a city profile (Y/N) "; City$
ELSE
  INPUT "   Do you want to see a city profile (Y/N) "; City$
END IF
CLS
IF UCASE$(City$) <> "N" THEN
  INPUT "   Press enter when ready for the city profile.                                ", X$
  CALL PlotSize(Xmin, Xmax, Ymin, Ymax)
  IF Scrn <> 11 THEN COLOR 3
 'Do Profile
  CALL Profile(Rows%, Cols%, Xmin, Xmax, Ymin, Ymax, Zmin, Zmax, Pop)
  CLS
END IF
END SUB

     'Contouring subroutines by Sam Outcalt
     'Converted to QuickBasic by Waldo Tobler
      SUB SetUpTri (Rows%, Cols%, H)
      DEFINT I-N
      SHARED Z(), Scrn
      IF Scrn <> 11 THEN COLOR 14
        FOR I = 2 TO Rows%
        K = Rows% - I + 1
        YSW = K
        YSE = YSW
        YNW = K + 1
        YNE = YNW
          FOR J = 2 TO Cols%
          XSW = J - 1
          XNW = XSW
          XSE = J
          XNE = XSE
          ZSW = Z(K, J - 1) + 100!
          ZSE = Z(K, J) + 100!
          ZNW = Z(K + 1, J - 1) + 100!
          ZNE = Z(K + 1, J) + 100!
          CALL TriCon(XSW, XNW, XNE, YSW, YNW, YNE, ZSW, ZNW, ZNE, H)
          CALL TriCon(XSW, XSE, XNE, YSW, YSE, YNE, ZSW, ZSE, ZNE, H)
          NEXT J
        NEXT I
      EXIT SUB
      END SUB

FUNCTION TimeDif$ (T1$, T2$)
OneMin = 1 / 60
Hr1 = VAL(MID$(T1$, 1, 2)) + VAL(MID$(T1$, 4, 2)) / 60
Hr2 = VAL(MID$(T2$, 1, 2)) + VAL(MID$(T2$, 4, 2)) / 60
IF Hr2 - Hr1 < OneMin THEN
  TimeDif$ = "Short"
ELSE
  TimeDif$ = "Long"
END IF
END FUNCTION

      SUB TriCon (X1, X2, X3, Y1, Y2, Y3, F1, F2, F3, H)
      DEFINT I-N
      DIM X(2), Y(2)
      Xa = X1
      XB = X2
      Xc = X3
      Ya = Y1
      YB = Y2
      Yc = Y3
      FA = F1
      FB = F2
      FC = F3
      CALL Order(Xa, XB, Xc, Ya, YB, Yc, FA, FB, FC)
      M = FIX(FC / H)
      N = FIX(FA / H) + 1
      ICON& = H * 5
      K = M - N + 1
      IF K <= 0 THEN GOTO AllDone
      FOR J = N TO M
        Fi = CSNG(J) * H
        BECK = Fi / ICON&
        DECK = BECK - FIX(BECK)
        IF FA = FC THEN GOTO GetNext
        X(1) = Xint(Xa, Xc, FA, FC, Fi)
        Y(1) = Xint(Ya, Yc, FA, FC, Fi)
        IF Fi > FB THEN X(2) = Xint(XB, Xc, FB, FC, Fi)
        IF Fi > FB THEN Y(2) = Xint(YB, Yc, FB, FC, Fi)
        IF Fi <= FB THEN X(2) = Xint(Xa, XB, FA, FB, Fi)
        IF Fi <= FB THEN Y(2) = Xint(Ya, YB, FA, FB, Fi)
        PSET (X(1), Y(1))
        LINE -(X(2), Y(2))
        IF DECK > 0! THEN GOTO GetNext
        PSET (X(1), Y(1))
        LINE -(X(2), Y(2))
GetNext:
     NEXT J
AllDone:
    EXIT SUB
    END SUB

      FUNCTION Xint (X1, X2, F1, F2, Fi)
      IF F1 = F2 THEN Xint = X1
      XRXINT = X1
      IF F1 = F2 THEN GOTO GotIt
      Xint = X1 + ((Fi - F1) / (F2 - F1)) * (X2 - X1)
      XRXINT = X1 + ((Fi - F1) / (F2 - F1)) * (X2 - X1)
GotIt:
      EXIT FUNCTION
      END FUNCTION

