DECLARE FUNCTION Min! (X!, Y!)
' Quadratic Transportation Problem
' Program QTP_PLUS.BAS
' W. Tobler 9/25/82,...,1/25/99
' MS QuickBasic 4.5
' tobler@geog.ucsb.edu

DEFINT I-N
OPTION BASE 1
DECLARE SUB Boundry (Bdy$)
DECLARE SUB Plotmap (Same$)
DECLARE FUNCTION Max! (a!, b!)
DECLARE SUB PlotFlow (Mbar!, Same$)
DECLARE FUNCTION TimeDif$ (T1$, T2$)
DECLARE SUB Arrow (X!(), Y!(), Ahl!)
DECLARE SUB PlusMins (Same$, Bdy$, T$)
DECLARE SUB NicePrint (NR%, NC%, Z!())
DECLARE SUB Symmetry (R%, Mbar!, M!())
DECLARE SUB Lagrange (Lp, Ir, Ot%, T2, Ep, Sh)
DECLARE SUB GetData (Px$, Dmt, XYform$, Bdy$, Same$, Dist$, DistPrint$)
COMMON NR%, NC%, OutSum&(), InSum&(), Po&(), Pi&(), D(), E(), R(), M!()
COMMON Expon, Ncoords, X(), Y(), Ib, Xb(), Yb(), Bds()

' Program variables:
' T$ = Your title
' D() = Distances
' Nr% = Number of rows
' Nc% = Number of columns
' M() = Estimated movements
' OutSum&() = Observed row marginals
' SumOuts&() = Computed row marginals
' InSum&() = Observed column marginals
' SumIns&() = Computed column marginals
' E() = Pulls (computed enticements)
' R() = Pushes (computed repulsions)
' Expon = Optional distance exponent
' P() = Optional Populations
' Px$ ="Y" if populations are used
' Dmt = 0 if modulation of flow by traffic is desired/used
' Bdy$ = "Y" if boundary coordinates are available
' Same$ = "Y" if rows & column places are the same -> Nr%=Nc%
' XYform$ = "Y" if XY coordinates read in; otherwise distance table used
' AddSub$ = No or Symbol or Arrow map
' Ncoords = Number of coordinates; Nr% or Nr% + Nc%
' IntSol% = 'true' if integer solution is desired
CLOSE
CLS
Fmt$ = "##.######^^^^"
PRINT
COLOR 14
PRINT "       QUADRATIC TRANSPORTATION PROBLEM"
PRINT
'The next print lines could be printed out and then deleted if you
'get tired of reading them.
PRINT "       Find movements M(i,j) to minimize the sum over all i,j of"
PRINT
PRINT "       M(i, j) ^ 2 * D(i, j) "
PRINT
PRINT "       when given the Outsums(i) and Insums(j), and distances D(i,j)."
PRINT
PRINT "       The resulting model is:"
PRINT
PRINT "       M(i,j) = {Pushes(i) + Pulls(j)} / D(i,j)."
PRINT
PRINT "       A map can be drawn if, and only if, XY coordinates are used"
PRINT "       to compute the distances. Two kinds of maps are available."
PRINT "       One shows by symbols the value of the move, the other shows"
PRINT "       by arrows the direction of movement, but only if the number"
PRINT "       of places is relatively small."
PRINT
PRINT "       A screen saver must be used to save map image. If you did"
PRINT "       not install a screen saver and wish to save maps, then exit"
PRINT "       the program now and install one."
PRINT
INPUT "       Press `enter' when ready or Ctrl+C to exit.", X$
CLS
PRINT
PRINT
PRINT "       Alternate models which can be estimated with the program are:"
PRINT
PRINT "       M(i,j) = {Pushes(i) + Pulls(J)} / D(i,j)^b, using an exponent,"
PRINT
PRINT "       or using a size variable, e.g., population:"
PRINT
PRINT "       M(i,j) = {Pushes(i) + Pulls(j)} * P(i) * P(j) / D(i,j)^b, or"
PRINT
PRINT "       using a conjestion variable, e.g., the flow quantities themselves:"
PRINT
PRINT "       M(i,j) = {Pushes(i) + Pulls(j)} * O(i) * I(j) / D(i,j)^b."
PRINT
PRINT
PRINT "       Copyright W. Tobler 1999"
PRINT
PRINT

'Don't delete these
INPUT "       Press `enter' when ready ", X$
PRINT
CLS
PRINT

'OK to delete, if you can remember it all
PRINT
PRINT " INPUT DATA:"
PRINT " Row Sums = sources, origins, supplies (required)."
PRINT " Column Sums = sinks, destinations, demands (required)."
PRINT " Source amounts to be entered first, then destination quantities."
PRINT " A vector of length rows + columns is expected."
PRINT " Origins and destinations need not, but may be, the same places."
PRINT " If they are, the number of rows = number of columns (a square table)."
PRINT
PRINT " Choice of distance exponent between 0 and 3; recommended is 1. "
PRINT
PRINT " If modulation by population is desired then a vector of these is required."
PRINT " A vector of length rows + columns is expected, and"
PRINT " then the model is M(i,j) = {Pushes(i) + Pulls(j)} * P(i) * P(j) / D(i,j)"
PRINT
PRINT " It is also possible to modulate the flows by the traffic, and"
PRINT " then the model is M(i,j) = {Pushes(i) + Pulls(j)} * O(i) * I(j) / D(i,j)"
PRINT
PRINT " A complete distance table (rows * columns in size) OR a vector of"
PRINT " coordinates is always required. This vector may be of length rows"
PRINT " when the places are the same or of rows plus columns in length"
PRINT " when the places are different."
PRINT

'Don't delete any more
INPUT " Press `enter' when ready ", X$
PRINT
COLOR 7
CLS
PRINT
LINE INPUT " Enter a title describing the data ", T$
PRINT
Another:
INPUT " Enter the name of the file to which you wish output sent:  ", OutFile$
'OutFile$ = "D:\QtpOut"      'For Testing
FOR I = 1 TO LEN(OutFile$)
  a$ = MID$(OutFile$, I, 1)
  IF a$ = ";" OR a$ = "," THEN
    PRINT " That is not a valid file name"
    BEEP
    BEEP
    GOTO Another
  END IF
NEXT I
OPEN "O", #4, OutFile$
PRINT #4, " Title: "; UCASE$(T$)
PRINT
PRINT " Enter the size of the tableau:"
MoreRows:
INPUT " Rows of data (from places)  "; NR%   'Number of FROM places
  IF NR% < 3 THEN
    PRINT " That's to small. Try a bigger set of data."
    BEEP
    GOTO MoreRows
  END IF
MoreCols:
INPUT " Columns of data (to places) "; NC%  'Number of TO places
IF NC% < 3 THEN
  PRINT " That's to small. Try a bigger set of data."
  BEEP
  GOTO MoreCols
END IF
PRINT
Rc% = NR%
IF NR% = NC% THEN
  PRINT " Your number of rows is equal to the number of columns."
  PRINT " Does this imply that the source places and the sink places"
  INPUT " are the same?  (Default Y)  Y/N ", Same$
  IF Same$ = "" THEN Same$ = "y"
  Same$ = UCASE$(Same$)
  PRINT
  IF Same$ = "Y" THEN
    Ncoords = NR%
  ELSE
    Ncoords = NR% + NC%
  END IF
ELSE
  IF NC% > NR% THEN Rc% = NC%
  Ncoords = NR% + NC%
END IF
PRINT
PRINT " When entering file names give the complete path"
PRINT " unless they are in the same directory as the program."
PRINT
INPUT " Enter the name of the file with the source and sink amounts   ", DataFile$
DoOverData:
FOR I = 1 TO LEN(DataFile$)
  a$ = MID$(DataFile$, I, 1)
  IF a$ = ";" OR a$ = "," THEN
    PRINT " That is not a valid file name"
    PRINT
    BEEP
    BEEP
    GOTO DoOverData
  END IF
NEXT I
OPEN "I", #1, DataFile$
 'The program also works for symmetric From/To tables. That is, with
 'source amounts (row sums) = sink amounts (column sums).
TryAgain:
PRINT
PRINT " Want a positive value less than 3 for an exponent? If yes enter it now."
INPUT " (The default value is 1;  `enter' yields default)   ", Expon
PRINT
IF Expon = 0 THEN Expon = 1
IF Expon < 0 OR Expon > 3 THEN
  IF Expon < 0 THEN PRINT " That is not a positive number."
  PRINT " Your entry is not valid. (0 < exponent < 3)  Try again."
  GOTO TryAgain
END IF
INPUT " Do you wish modulation by population? (Default N)  N/Y ", Px$
Px$ = UCASE$(Px$)
IF Px$ = "" THEN Px$ = "N"
IF Px$ <> "Y" THEN GOTO NoPop
INPUT " Enter the name of the file containing population values  ", Pop$
OPEN "I", #2, Pop$
NoPop:
INPUT " Do you wish to use a vector of coordinates? (Default N) N/Y ", XYform$
XYform$ = UCASE$(XYform$)
IF XYform$ = "" THEN XYform$ = "N"
IF XYform$ <> "Y" THEN  '-> reads distance table
  PRINT
  INPUT " Enter the name of the file containing the distance table ", Dist$
ELSE     'If XYform$="Y" -> will use coordinates
  PRINT
  PRINT " The program will expect"; Ncoords; "pairs of x,y values."
DoOverDist:
  INPUT " Enter the name of the file containing these coordinates  ", Dist$
  FOR I = 1 TO LEN(Dist$)
    a$ = MID$(Dist$, I, 1)
    IF a$ = ";" OR a$ = "," THEN
      PRINT " That is not a valid file name"
      PRINT
      BEEP
      BEEP
      GOTO DoOverDist
    END IF
  NEXT I
  OPEN "I", #3, Dist$
  PRINT
  INPUT " Do you have a boundary file Y/N  "; Bdy$
  Bdy$ = UCASE$(Bdy$)
  IF Bdy$ = "" THEN Bdy$ = "Y"
  IF UCASE$(Bdy$) = "Y" THEN
DoOver3:
    INPUT " Enter name of the boundary coordinate file: "; BdyIn$
    FOR I = 1 TO LEN(BdyIn$)
      a$ = MID$(BdyIn$, I, 1)
      IF a$ = ";" OR a$ = "," THEN
        PRINT " That is not a valid file name"
        PRINT
        BEEP
        BEEP
        GOTO DoOver3
      END IF
    NEXT I
    IF Px$ <> "Y" THEN
      OPEN "I", #2, BdyIn$
    ELSE
      OPEN "I", #5, BdyIn$
    END IF
    PRINT
  END IF
END IF
INPUT " Do you wish to output impedance table? (Default N) N/Y ", DistPrint$
IF DistPrint$ = "" THEN DistPrint$ = "n"
DistPrint$ = UCASE$(DistPrint$)
IntSol% = 0
INPUT " Do you wish to force an integer solution? (Default N)  N/Y ", Z$
Z$ = UCASE$(Z$)
IF Z$ = "" THEN Z$ = "N"
IF Z$ = "Y" THEN IntSol% = -1
Dmt = 1  'Distances modulated by traffic
IF Px$ = "Y" THEN GOTO PopMod
'Modulation by both population and traffic is not permitted.
INPUT " Should distances be modulated by flow magnitudes? (Default N) N/Y ", Xx$
Xx$ = UCASE$(Xx$)
IF Xx$ = "" THEN Xx$ = "N"
IF Xx$ = "Y" THEN Dmt = 0
PopMod:
PRINT
PRINT " Rows = "; NR%; " Columns = "; NC%
PRINT
INPUT " Is this ok? (Default Y) Y/N ", X$
PRINT
X$ = UCASE$(X$)
IF X$ = "" THEN X$ = "Y"
IF X$ = "Y" THEN
  GOTO AllOK
ELSE
  PRINT Bad$
  CLOSE
  END
END IF
AllOK:
DIM SumOuts&(NR%), OutSum&(NR%), SumIns&(NC%), InSum&(NC%), Po&(NR%), Pi&(NC%)
IF XYform$ = "Y" THEN DIM X(Ncoords), Y(Ncoords)
DIM M!(NR%, NC%), D(NR%, NC%)
'Read the data now
CALL GetData(Px$, Dmt, XYform$, Bdy$, Same$, Dist$, DistPrint$)
PRINT " Starting to compute. Patience is a virtue."
MaxSize% = NR%
IF NC% > NR% THEN MaxSize% = NC%
MaxSize% = MaxSize% + 1
PRINT " Expect at least"; MaxSize%; "iterations."
Now1$ = MID$(TIME$, 1, 5)
PRINT " Iteration number ";
DIM R(NC%), E(NR%), R1(NC%), E1(NR%)
'Initialize values
FOR K% = 1 TO NR%: E(K%) = 1: E1(K%) = 0: NEXT K%
FOR K% = 1 TO NC%: R1(K%) = 0: NEXT K%
Lp = 1000: Ir = 0: Ot% = 0: T2 = 2: Ep = 12: Sh = .5
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
DoSomeMore:
Ts = 1
'Get a trial solution
CALL Lagrange(Lp, Ir, Ot%, T2, Ep, Sh)
Ot% = Ot% + 1: T = 1E+32
PRINT Ot%;
'Check solution
'Along rows
FOR K% = 1 TO NR%
  IF E(K%) > T THEN GOTO EBig
  T = E(K%)
EBig:
NEXT K%
'Along columns
FOR J% = 1 TO NC%
  SumIns&(J%) = 0
  R(J%) = R(J%) + T
NEXT J%
'Now rows again
FOR K% = 1 TO NR%
  SumOuts&(K%) = 0
  E(K%) = E(K%) - T
NEXT K%
F1 = 1: T = 0: Df = 0: T3 = 0: S = 0: S1 = 0
'Clear out last estimate
FOR K% = 1 TO NR%
  FOR J% = 1 TO NC%
    M!(K%, J%) = 0
  NEXT J%
NEXT K%
'Compute new estimate from push-pull values
FOR K% = 1 TO NR%
  FOR J% = 1 TO NC%
    Sm = (R(J%) + E(K%)) * D(K%, J%)
    Sm = Sm * Sh
    M!(K%, J%) = Sm
    SumIns&(J%) = SumIns&(J%) + Sm
    SumOuts&(K%) = SumOuts&(K%) + Sm
    IF Sm >= 0 THEN GOTO NotNeg
   'Zero out negative movement quantities
    F1 = 0
    D(K%, J%) = 0
    NegCount% = NegCount% + 1
NotNeg:
  NEXT J%
NEXT K%
'Sum difference between observed and estimated
'Outsums
Df = 0
FOR K% = 1 TO NR%
  Df = Df + ABS(OutSum&(K%) - SumOuts&(K%))
NEXT K%
'and Insums
FOR J% = 1 TO NC%
  Df = Df + ABS(InSum&(J%) - SumIns&(J%))
NEXT J%
'Done if no negatives remain and R*C iterations have been performed.
IF F1 > 0 AND Ot% > Rc% THEN GOTO AllDoneNow
GOTO DoSomeMore   ' More iterations necessary
'  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
AllDoneNow:
FOR K% = 1 TO NR%
  SumOuts&(K%) = 0
  FOR J% = 1 TO NC%
    SumIns&(J%) = 0
    M!(K%, J%) = 0
  NEXT J%
NEXT K%
Kk = 0
FOR K% = 1 TO NR%
  FOR J% = 1 TO NC%
    Sm = (R(J%) + E(K%)) * D(K%, J%)
    Sm = Sm * Sh                 'Sh=0.5, Lagrangian constant
    IF NOT IntSol% THEN GOTO FracSol
   'Next code line forces an integer solution.
   'Push, pull, & functional values will be slightly different
   'for the integer solution.
    Sm = INT(Sm + Sh)
FracSol:
    M!(K%, J%) = Sm
    IF M!(K%, J%) > 0 THEN Kk = Kk + 1
    SumIns&(J%) = SumIns&(J%) + Sm
    SumOuts&(K%) = SumOuts&(K%) + Sm
    IF D(K%, J%) <> 0 THEN
      T = M!(K%, J%) / D(K%, J%)   'Gets a linear cost value
      T3 = M!(K%, J%) * T          'Gets the quadratic cost value
    END IF
  NEXT J%
NEXT K%
'Done, send results to output file
IF DistPrint$ <> "N" THEN
  PRINT #4, " The table of estimated flows"
  PRINT #4, " is"; NR%; "rows by"; NC%; "columns in size."
  PRINT #4,
END IF
PRINT
PRINT #4, " The table of estimated flows is:"
CALL NicePrint(NR%, NC%, M!())
PRINT #4,
IF DistPrint$ = "N" THEN
  PRINT #4, " The above table of estimated flows"
  PRINT #4, " is"; NR%; "rows by"; NC%; "columns in size."
END IF
'Compute and print degree of asymmetry of M() here.
IF NR% = NC% AND Same$ = "Y" THEN
  CALL Symmetry(NR%, Mbar!, M!())
END IF
Df3 = 0
RowMax% = 0
FOR K% = 1 TO NR%              'Do all the rows
  S2 = 0
  FOR J% = 1 TO NC%            'By column
    IF D(K%, J%) <= 0 THEN GOTO DontDivide
    S = M!(K%, J%) / D(K%, J%)
    T = T + S                  'Gets a linear cost value
    T3 = T3 + M!(K%, J%) * S   'Gets the quadratic cost value
DontDivide:
    S2 = S2 + M!(K%, J%)
    S1 = S1 + M!(K%, J%)    'Used to find average
  NEXT J%
  Df1 = 0
  IF K% = 1 THEN
    Df = 0
    PRINT #4,
    PRINT #4, NR%; "Row sums:     Estimate    Original   Difference"
  END IF
  S2Sum = S2Sum + S2
  Osum& = Osum& + OutSum&(K%)
  IF (Df1 <= ABS(S2 - OutSum&(K%))) THEN
    Df1 = ABS(S2 - OutSum&(K%))
    IF Df1 < .0001 THEN Df1 = 0
    IF Df1 > 0 AND Df1 > Df3 THEN
      RowMax% = K%
      Df = Max(Df1, Df)
      Df3 = Df1
    END IF
  END IF
  PRINT #4, K%; "           ";
  PRINT #4, USING Fmt$; S2;      'This is the estimate
  PRINT #4, "   "; OutSum&(K%);       'This is the original
  PRINT #4, "         "; Df1     'This is the difference
NEXT K%
PRINT #4,
PRINT #4, " Totals are   ";
PRINT #4, USING Fmt$; S2Sum;
PRINT #4, "   "; Osum&
IF Df > 0 THEN
  PRINT " Maximum difference between observed and estimated row sum"
  PRINT " is"; Df; "at row"; RowMax%
  PRINT #4, " Maximum difference between observed and estimated row sum"
  PRINT #4, " is"; Df; "at row"; RowMax%
  PRINT #4, " Total difference is"; ABS(S2Sum - Osum&)
  Df1 = Df
END IF
Df = 0
ColMax% = 0
Df3 = 0
PRINT #4,
PRINT #4, NC%; "Column sums:  Estimate    Original   Difference"
FOR J% = 1 TO NC%          'Now do the columns
  S = 0
  Df2 = 0
  FOR K% = 1 TO NR%        'By row
    S = S + M!(K%, J%)
  NEXT K%
  SumS = SumS + S
  SumI = SumI + InSum&(J%)
  IF (Df2 <= ABS(S - InSum&(J%))) THEN
    Df2 = ABS(S - InSum&(J%))
    IF Df2 < .0001 THEN Df2 = 0
    IF Df2 > 0 AND Df2 > Df3 THEN
      ColMax% = J%
      Df = Max(Df2, Df)
      Df3 = Df2
    END IF
  END IF
  PRINT #4, J%; "           ";
  PRINT #4, USING Fmt$; S;          'Estimated column sum
  PRINT #4, "   "; InSum&(J%);      'Original column sum
  PRINT #4, "         "; Df2        'Their difference
NEXT J%
PRINT #4,
PRINT #4, " Totals are   ";
PRINT #4, USING Fmt$; SumS;
PRINT #4, "   "; SumI
IF Df2 > 0 THEN
  PRINT " Maximum difference between observed and estimated column sum"
  PRINT " is"; Df; "at column"; ColMax%
  PRINT #4, " Maximum difference between observed and estimated column sum"
  PRINT #4, " is"; Df; "at column"; ColMax%
  PRINT #4, " Total difference is"; ABS(SumS - SumI)
  Df2 = Df
END IF
Df = Max!(Df1, Df2)
IF NR% = NC% AND Same$ = "Y" THEN
  PRINT #4,
  PRINT #4, "      The computed potentials are as follows:"
  PRINT #4, "      Pushes        Pulls         Attractivity  Turnover"
  FOR K% = 1 TO NR%
    PRINT #4, K%; "  ";
    PRINT #4, USING Fmt$; E(K%);
    PRINT #4, " ";
    PRINT #4, USING Fmt$; R(K%);
    PRINT #4, " ";
    PRINT #4, USING Fmt$; E(K%) + R(K%);
    PRINT #4, " ";
    PRINT #4, USING Fmt$; E(K%) - R(K%)
  NEXT K%
  PRINT #4,
  PRINT #4, " The attractivity is the sum of the pushes and pulls."
  PRINT #4, " The turnover is computed as the pushes minus the pulls."
ELSE
  PRINT #4,
  PRINT #4, " Pushes  -  associated with the"; NR%; "origins."
  FOR K% = 1 TO NR%
    PRINT #4, K%; "  ";
    PRINT #4, USING Fmt$; E(K%)
  NEXT K%
  PRINT #4,
  PRINT #4, " Pulls  -  associated with the"; NC%; "destinations."
  FOR J% = 1 TO NC%
    PRINT #4, J%; "  ";
    PRINT #4, USING Fmt$; R(J%)
  NEXT J%
END IF
PRINT
PRINT
PRINT " D O N E"
BEEP: BEEP
PRINT
PRINT " Title: ";
COLOR 14
PRINT UCASE$(T$)
COLOR 7
IF Expon = 1 THEN
  PRINT " The estimated transport effort times movement volumes:"
  PRINT " For the quadratic program the minimized sum of m^2 * d is"; T3
  PRINT " The sum of m*d is"; T; "(& exceeds the linear programming solution)"
  IF ABS(T - T3) < .0001 THEN
    PRINT " These totals can be equal iff the movements on each link are one"
  END IF
END IF
PRINT
IF XYform$ = "Y" THEN      'Can't show map without coordinates
  PRINT
  INPUT " Do you want to see a map? (Default Y, but it's a bit slow) (Y/N) ", Show$
  INPUT " Do you wish a map of symbols or of arrows N/S/A "; AddSub$
  IF AddSub$ = "" THEN AddSub$ = "N"
  AddSub$ = UCASE$(AddSub$)
  IF Show$ = "" OR Show$ = "y" THEN Show$ = "Y"
  IF Show$ = "Y" THEN
   'Prepare for map
    FOR J% = 1 TO Count%
      IF X(J%) > Xmax THEN Xmax = X(J%)
      IF X(J%) < Xmin THEN Xmin = X(J%)
      IF Y(J%) > Ymax THEN Ymax = Y(J%)
      IF Y(J%) < Ymin THEN Ymin = Y(J%)
    NEXT J%
    Dx = Xmax - Xmin: Dy = Ymax - Ymin
    IF Dy > Dx THEN Dx = Dy
    Xmin = Xmin - .3 * Dx: Xmax = Xmax + .3 * Dx
    Ymin = Ymin - .3 * Dx: Ymax = Ymax + .3 * Dx
    CLS
    CALL Boundry(Bdy$)
    IF AddSub$ = "N" THEN GOTO SkipMap
    IF AddSub$ = "Y" THEN
      CALL PlusMins(Same$, Bdy$, T$)
    ELSE
      IF MaxSize% > 10 THEN
        PRINT " Too many locations to do arrow map"
        GOTO SkipMap
       'Can't show massive number of arrows with this program.
      END IF
      CALL PlotFlow(Mbar!, Same$)
    END IF
SkipMap:
  END IF
END IF
PRINT #4,
IF Expon = 1 THEN
  PRINT #4, " The estimated transport effort times movement volumes:"
  PRINT #4, " For the quadratic program the minimized sum of m^2 * d is"; T3
  PRINT #4, " The sum of m * d is"; T; "(& exceeds the linear programming solution)"
  IF ABS(T - T3) < .0001 THEN
    PRINT #4, " These totals are equal iff the movements on each link are one"
  END IF
  PRINT #4, " Average distance traveled is"; T / S1; "units."
END IF
PRINT #4,
PRINT #4, " Source, sink amounts came from "; UCASE$(DataFile$)
IF XYform$ = "Y" THEN
  PRINT #4, " Coordinates came from "; UCASE$(Dist$)
  PRINT #4, " and distances were computed from these coordinates."
 'PRINT #4, " Xmin="; Xmin + .3 * Dx; "Xmax="; Xmax - .3 * Dx; "Ymin="; Ymin + .3 * Dx; "Ymax="; Ymax - .3 * Dx
  IF Bdy$ = "Y" THEN
    PRINT #4, " Boundary came from "; UCASE$(BdyIn$)
  ELSE
    PRINT #4, " No boundary file was available."
  END IF
ELSE
  PRINT #4, " Distances came from "; UCASE$(Dist$)
END IF
IF Px$ = "Y" THEN
  PRINT " Distances modulated by population."
  PRINT #4, " Distances modulated by population."
  PRINT #4, " Population values came from "; UCASE$(Pop$)
ELSE
  PRINT " Distances not modulated by population."
  PRINT #4, " Distances not modulated by population."
  IF Dmt = 0 THEN PRINT " Distances modulated by flows."
  IF Dmt = 0 THEN PRINT #4, " Distances modulated by flows."
  IF Dmt = 1 THEN PRINT " Distances not modulated by flows."
  IF Dmt = 1 THEN PRINT #4, " Distances not modulated by flows."
END IF
NoDist:
  PRINT #4, " The exponent used was"; Expon
  PRINT " The exponent used was"; Expon
  IF Same$ = "Y" THEN
    Flows% = NR% ^ 2 - NR%
  ELSE
    Flows% = NR% * NC%
  END IF
IF Df < 1 THEN GOTO NonDiscrp
PRINT " Largest discrepancy between observed and estimated sum is"; Df
PRINT #4, " Largest discrepancy between observed and estimated sum is"; Df
NonDiscrp:
PRINT #4, " There were"; Kk; "movement paths (out of"; Flows%; "possible)"
PRINT #4, " with an estimated total movement of"; S2Sum
PRINT #4, " The actual total movement was"; Osum&
PRINT " There were"; Kk; "movement paths (out of"; Flows%; "possible)"
PRINT " with an estimated total movement of"; S2Sum
PRINT " The actual total movement was"; Osum&
IF IntSol% THEN PRINT " Integer solution forced."
IF IntSol% THEN PRINT #4, " Integer solution forced."
IF NOT IntSol% THEN PRINT " Integer solution not forced."
IF NOT IntSol% THEN PRINT #4, " Integer solution not forced."
IF NegCount% <> 0 THEN
  IF NegCount% = 1 THEN
    PRINT " One negative quantity encountered during iterations"
   'PRINT " and one distance set to zero."
  ELSE
    PRINT " Negative quantities encountered during iterations"
   'PRINT " and;"NegCount%; "distances set to zero."
  END IF
ELSE
  PRINT " No negative quantities appeared during iterations."
END IF
PRINT #4, " The number of iterations needed was"; Ot%
IF NegCount% <> 0 THEN
  IF NegCount% = 1 THEN
    PRINT #4, " One negative quantity encountered during iterations"
    PRINT #4, " and one distance set to zero."
  ELSE
    PRINT #4, " Negative quantities encountered during iterations"
    PRINT #4, " and"; NegCount%; "distances set to zero."
  END IF
ELSE
  PRINT #4, " No negative quantities appeared during iterations."
END IF
'Print a description of the model
PRINT #4,
PRINT #4, " The model is M(i,j) = ( Pushes(i) + Pulls(j) )";
IF Px$ = "Y" THEN PRINT #4, " * P(i) * P(j)";
IF DistModByTraffic = 0 THEN PRINT #4, " * O(i) * I(j)";
IF Expon = 1 THEN
  PRINT #4, " / D(i,j)"
ELSE
  PRINT #4, " / D(i,j) ^"; Expon
END IF
PRINT #4,
PRINT #4, " This is file "; UCASE$(OutFile$)
PRINT #4, " Quadratic Transportation Problem Solution"
PRINT #4, " Program QTP.BAS, version of May 3 2003"
PRINT #4, " by Waldo Tobler"
Now2$ = MID$(TIME$, 1, 5)
IF TimeDif$(Now1$, Now2$) <> "Short" THEN
  PRINT
  PRINT " Started at  "; Now1$; " on "; DATE$; "."
  PRINT #4,
  PRINT #4, " Computation started at  "; Now1$; " on "; DATE$; "."
END IF
PRINT " Finished at "; Now2$; " on "; DATE$; "."
PRINT #4, " Computation finished at "; Now2$; " on "; DATE$; "."
CLOSE
PRINT
PRINT " Look at your output file for the values of the `Pushes and Pulls'"
PRINT " and for the table of estimated movements. Your output file is:"
PRINT
COLOR 14
PRINT " "; UCASE$(OutFile$)
COLOR 12
PRINT
PRINT
PRINT " THANK WALDO TOBLER FOR THIS PROGRAM"
PRINT
PRINT
COLOR 3
PRINT " Please press `enter' to end."
DO WHILE INKEY$ = "": LOOP
COLOR 7
Done:
END

DEFSNG I-N
      SUB Arrow (X(), Y(), Ahl)
      DEFINT I-N
      Pi = 3.14159265#
     'Draw arrowhead first, with vertex angle of Beta
      Beta = 15 / 2 * (Pi / 180) 'convert to radians
     'Set arrowhead corners relative to point, preparatory to rotation
      Xa = -Ahl
      Ya = Ahl * TAN(Beta)
      Dx = X(2) - X(1)
      Dy = (Y(2) - Y(1)) * 4 / 3
     'PRINT #4, " Arrowhead length is"; Ahl
      IF ((Dy = 0!) AND (Dx = 0!)) THEN EXIT SUB
      IF Dx <> 0 THEN
        Theta = ATN(Dy / Dx)
        IF Dx < 0 THEN
          IF Dy > 0 THEN   'Dx<0 & Dy>0
            Theta = Theta + Pi
          ELSE             'Dx<0 & Dy<0
            Theta = Theta + Pi
          END IF
        END IF
      ELSE                 'Dx=0
        IF Dy < 0 THEN Theta = -Pi / 2
        IF Dy > 0 THEN Theta = Pi / 2
      END IF
      Theta = -Theta
      Sa = SIN(Theta)
      Ca = COS(Theta)
     'Rotate the arrowhead corners counterclockwise about the vertex
      X1 = Ca * Xa + Sa * Ya
      Y1 = -Sa * Xa + Ca * Ya
      Ya = -Ya
      X2 = Ca * Xa + Sa * Ya
      Y2 = -Sa * Xa + Ca * Ya
     'and now translate them to the head of the arrow.
      X1 = X1 + X(2): Y1 = (Y1 + Y(2)) * 4 / 3: X2 = X2 + X(2): Y2 = (Y2 + Y(2)) * 4 / 3
      PSET (X(2), Y(2))
      LINE -(X1, Y1), 14
      LINE -(X2, Y2), 14
      LINE -(X(2), Y(2)), 14
      Cx = (X1 + X2 + X(2)) / 3: Cy = (Y1 + Y2 + (Y(2) * 4 / 3)) / 3
      PAINT (Cx, Cy), 14     'Fill the arrowhead with a solid color
     'Now draw the tail to the origin X,Y of the arrow
      LINE -(X(1), Y(1) * 4 / 3), 14
      END SUB

DEFSNG I-N
SUB Boundry (Bdy$)
'Plot boundary
DEFINT I-N
SHARED Expon, Ncoords, X(), Y(), Ib, Xb(), Yb(), Bds()
IF UCASE$(Bdy$) <> "N" THEN
  PSET (Xb(1), Yb(1))
  FOR I = 2 TO Ib
    LINE -(Xb(I), Yb(I))
  NEXT I
ELSE
' Do a rectangular border if there is no boundary
  Xmin = Bds(1): Ymin = Bds(2): Xmax = Bds(3): Ymax = Bds(4)
  GOTO GotEm
  Xmax = -1E+09: Xmin = -Xmax: Ymax = -1E+09: Ymin = -Ymax
  FOR I = 2 TO Ncoords
    Ys = Y(I) * 4 / 3
    Xmax = Max!(X(I), Xmax)
    Xmin = Min!(X(I), Xmin)
    Ymax = Max!(Ys, Ymax)
    Ymin = Min!(Ys, Ymin)
  NEXT I
  Dx = Xmax - Xmin: Dy = Ymax - Ymin
  Range = Max(Dx, Dy)
  Xmin = Xmin - .05 * Range
  Xmax = Xmin + 1.1 * Range
  Ymin = Ymin - .05 * Range
  Ymax = Ymin + 1.1 * Range
GotEm:
  PSET (Xmin, Ymin)
  LINE -(Xmax, Ymin)
  LINE -(Xmax, Ymax)
  LINE -(Xmin, Ymax)
  LINE -(Xmin, Ymin)
END IF
END SUB

SUB GetData (Px$, Dmt, XYform$, Bdy$, Same$, Dist$, DistPrint$)
SHARED NR%, NC%, OutSum&(), InSum&(), Po&(), Pi&(), D()
SHARED Expon, Ncoords, X(), Y(), Ib, Xb(), Yb(), Bds()
DIM Ys(NR% * NC%)
Fmt$ = "##.######^^^^"
OPEN "I", #3, Dist$
'READ DATA FILES
'Sources = supplies = outsums = origins = rows (FROM quantities)
Sm = 0
FOR K% = 1 TO NR%
  INPUT #1, OutSum&(K%)
  Sm = Sm + OutSum&(K%)
NEXT K%
'Sinks = demands = insums = destinations = columns (TO quantities)
FOR K% = 1 TO NC%
  INPUT #1, InSum&(K%)
  Sm = Sm - InSum&(K%)
NEXT K%
IF Sm = 0 THEN GOTO SumOK
PRINT " Summation error - in and out quantities do not match exactly."
PRINT " Difference is: "; Sm
PRINT
INPUT " Is this ok? (Default Y) Y/N ", X$
PRINT
IF X$ = "y" OR X$ = "Y" OR X$ = "" THEN
  GOTO SkipSum
ELSE
  PRINT Bad$
  CLOSE
  END
END IF
GOTO SkipSum
SumOK:
PRINT " Summation check ok. In and out quantities match."
PRINT
SkipSum:
'Now get populations, if there are any.
'Observe that both row populations and column populations must be read in,
'even though these may be the same (when Rows=Columns). This is a single
'column vector of length rows + columns.
IF Px$ = "Y" THEN
  FOR K% = 1 TO NR%
    INPUT #2, Po&(K%)
  NEXT K%
  FOR K% = 1 TO NC%
    INPUT #2, Pi&(K%)
  NEXT K%
END IF
'DistModByTraffic = Dmt = 1 use normal distances
'DistModByTraffic = Dmt = 0 to modulate distances by flow size
Count% = 0
DIM Bds(4)
'Get distances
IF XYform$ <> "Y" THEN
 'Read distance table
  FOR K% = 1 TO NR%
    FOR J% = 1 TO NC%
      Count% = Count% + 1
      D(K%, J%) = 0
      INPUT #3, T
      IF T = 0 THEN
        Count% = Count% - 1
        GOTO ZeroDistance
      END IF
      T = T ^ Expon
      D(K%, J%) = 1 / T
      IF Px$ = "Y" THEN   'Modulate distance by population
        D(K%, J%) = D(K%, J%) * Po&(K%) * Pi&(J%)
        GOTO ZeroDistance
      END IF
      IF Dmt = 1 THEN GOTO ZeroDistance
      D(K%, J%) = D(K%, J%) * OutSum&(K%) * InSum&(J%) 'Modulate distance by traffic
ZeroDistance:
    NEXT J%
  NEXT K%
 'PRINT
  PRINT Count%; "non-zero distances read in."
ELSE
'Or get coordinates and compute distances, and perhaps a map
Xmax = -1E+32: Ymax = Xmax: Xmin = -Xmax: Ymin = Xmin
Count% = 0
  FOR K% = 1 TO Ncoords
   'Read vector of coordinates
    Count% = Count% + 1
    INPUT #3, X(Count%), Y(Count%)
    Y(Count%) = Y(Count%)
    Ys(Count%) = Ys(Count%) * 4 / 3     ' Adjust for screen shape
  NEXT K%
  PRINT Count%; "Pairs of coordinates are in now."
  IF UCASE$(Bdy$) <> "Y" THEN GOTO NoBdy
' Read the boundary
' The first value specifies the number of boundary coordinates
  IF Px$ = "Y" THEN
    INPUT #5, Ib
  ELSE
    INPUT #2, Ib
  END IF
  DIM Xb(Ib), Yb(Ib)
  FOR I = 1 TO Ib
    IF Px$ = "Y" THEN
      INPUT #5, Xb(I), Yb(I)
    ELSE
      INPUT #2, Xb(I), Yb(I)
    END IF
    Yb(I) = Yb(I) * 4 / 3         ' Adjust for screen shape
    IF Xb(I) > Xmax THEN Xmax = Xb(I)
    IF Yb(I) > Ymax THEN Ymax = Yb(I)
    IF Xb(I) < Xmin THEN Xmin = Xb(I)
    IF Yb(I) < Ymin THEN Ymin = Yb(I)
  NEXT I
NoBdy:
  INPUT " Do you wish to see Source & Sink location magnitudes Y/N "; Show$
  IF Show$ = "" OR Show$ = "y" THEN Show$ = "Y"
 'Prepare for map
  FOR J% = 1 TO Count%
    IF X(J%) > Xmax THEN Xmax = X(J%)
    IF X(J%) < Xmin THEN Xmin = X(J%)
    IF Ys(J%) > Ymax THEN Ymax = Ys(J%)
    IF Ys(J%) < Ymin THEN Ymin = Ys(J%)
  NEXT J%
  Dx = Xmax - Xmin: Dy = Ymax - Ymin
  IF Dy > Dx THEN Dx = Dy
  Xmin = Xmin - .3 * Dx: Xmax = Xmax + .3 * Dx
  Ymin = Ymin - .3 * Dx: Ymax = Ymax + .3 * Dx
  Bds(1) = Xmin: Bds(2) = Ymin: Bds(3) = Xmax: Bds(4) = Ymax
  SCREEN 12
  WINDOW (Xmin - .2 * Dx, Ymin - .2 * Dx)-(Xmax + .2 * Dx, Ymax + .2 * Dx)
  IF Show$ = "Y" THEN
    CLS
    CALL Boundry(Bdy$)
    CALL PlusMins(Same$, Bdy$, T$)
  END IF
  COLOR 7
  CLS
  Count% = 0
  IF NR% = NC% THEN
    IF Same$ <> "Y" THEN
      Nd% = NR% + NC%
      Ns% = NR% + 1
    ELSE
      Nd% = NC%
      Ns% = 1
    END IF
  ELSE
    Ns% = NR% + 1
    Nd% = NR% + NC%
  END IF
  FOR K% = 1 TO NR%             ' Compute distances from coordinates
    JJ% = 0
    FOR J% = Ns% TO Nd%
      JJ% = JJ% + 1
      Count% = Count% + 1
      T = SQR((X(K%) - X(J%)) ^ 2 + (Y(K%) - Y(J%)) ^ 2)
      T = T ^ Expon
      IF T <= 0 THEN
        Count% = Count% - 1
        GOTO NoDiv
      END IF
      D(K%, JJ%) = 1 / T
      IF Px$ = "Y" THEN          ' Modulate distance by population
        D(K%, JJ%) = D(K%, JJ%) * Po&(K%) * Pi&(JJ%)
        GOTO NoDiv
      END IF
      IF DistModByTraffic = 1 THEN GOTO NoDiv  'Modulate distance by traffic
      D(K%, JJ%) = D(K%, JJ%) * OutSum&(K%) * InSum&(JJ%)
NoDiv:
    NEXT J%
  NEXT K%
  PRINT
  PRINT Count%; "Non-zero distances calculated."
END IF          ' End of coordinate input
IF NR% <> NC% THEN
  PRINT NR% * NC%; "expected since the places are different."
ELSE
  PRINT NR% * NR% - NR%; "expected if the row, column places are the same."
  PRINT " but"; NR% * NC%; "expected if the places are different."
END IF
PRINT
INPUT " Is this ok? (Default Y) Y/N ", X$
IF X$ = "y" OR X$ = "Y" OR X$ = "" THEN
  PRINT
  GOTO GoodData
ELSE
  PRINT Bad$
  Bad$ = " Something's wrong. Program stopped by user."
  CLOSE
  END
END IF
GoodData:
PRINT " All data are in."
IF DistPrint$ = "N" THEN
  PRINT #4, " First three impedances - as a check - are:"
  IF D(1, 1) > 0 THEN PRINT #4, 1 / D(1, 1);
  IF D(1, 2) > 0 THEN PRINT #4, 1 / D(1, 2);
  IF D(1, 3) > 0 THEN PRINT #4, 1 / D(1, 3)
  PRINT #4,
END IF
IF DistPrint$ <> "N" THEN
  PRINT #4, " Table of impedances used was:"
  PRINT #4,
  FOR K% = 1 TO NR%
    FOR J% = 1 TO NC%
      IF D(K%, J%) <> 0 THEN D(K%, J%) = 1 / D(K%, J%)
    NEXT J%
  NEXT K%
  CALL NicePrint(NR%, NC%, D())
  IF XYform$ = "Y" THEN
    PRINT #4,
    PRINT #4, " Coordinates used were:"
    FOR K% = 1 TO Ncoords
      PRINT #4, K%;
      PRINT #4, USING Fmt$; X(K%); Y(K%)
    NEXT K%
    PRINT #4,
  END IF
END IF
ERASE Ys
END SUB

SUB Lagrange (Lp, Ir, Ot%, T2, Ep, Sh)
SHARED NR%, NC%, OutSum&(), InSum&(), Po&(), Pi&(), D(), E(), R(), M!()
'MAIN ITERATION FOR LAGRANGIANS
DIM R1(NC%), E1(NR%)
'Pulls associated with sinks
'Pushes associated with sources
FOR L = 1 TO Lp
  SS = Ts: Ts = 0
 'Estimate push (R)
  FOR J% = 1 TO NC%
    T = 0: S0 = 0
    FOR K% = 1 TO NR%
      S0 = S0 + D(K%, J%)
      T = T + E(K%) * D(K%, J%)
    NEXT K%
    IF S0 <> 0 THEN
      R(J%) = (T2 * InSum&(J%) - T) / S0
    END IF
  NEXT J%
'Estimate pull (E)
  FOR K% = 1 TO NR%
    T = 0: S0 = 0
    FOR J% = 1 TO NC%
      S0 = S0 + D(K%, J%)
      T = T + R(J%) * D(K%, J%)
    NEXT J%
    IF S0 <> 0 THEN
      E(K%) = (T2 * OutSum&(K%) - T) / S0
    END IF
  NEXT K%
'Now check convergence
Ir = Ir + 1: T = 0
'Compare to previous pull
FOR K% = 1 TO NR%
  Df = ABS(E(K%) - E1(K%))
  E1(K%) = E(K%)
  IF ABS(E(K%)) < Ts THEN GOTO Ebigger
    Ts = ABS(E(K%))
Ebigger:
    IF Df < SS THEN GOTO Smaller
    IF Df < T THEN GOTO Smaller
    T = Df
Smaller:
  NEXT K%
 'Compare to previous push
  FOR J% = 1 TO NC%
    Df = ABS(R(J%) - R1(J%))
    R1(J%) = R(J%)
    IF ABS(R(J%)) < Ts THEN GOTO Rbig
    Ts = ABS(R(J%))
Rbig:
    IF Df < SS THEN GOTO DfSmall
    IF Df < T THEN GOTO DfSmall
    T = Df
DfSmall:
  NEXT J%
  IF T = 0 THEN GOTO ThatsIt
  Ts = Ts * 10 ^ -Ep
NEXT L
ThatsIt:
END SUB

FUNCTION Max! (a, b)
Max! = a
IF b > a THEN Max! = b
EXIT FUNCTION
END FUNCTION

FUNCTION Min! (X, Y)
Min! = X
IF Y < X THEN Min! = Y
END FUNCTION

DEFINT C, F, R
DEFSNG K-N
  SUB NicePrint (R, C, Z())
  Five = 5
  Fmt$ = "##.######^^^^"
  FOR I = 1 TO R
    ColCount = 0
    FOR J = 1 TO C
      IF J < C THEN
        ColCount = ColCount + 1
        IF ColCount < Five THEN
          PRINT #4, USING Fmt$; Z(I, J);
          PRINT #4, " ";
        ELSE
          PRINT #4, USING Fmt$; Z(I, J)
          ColCount = 0
        END IF
      ELSE
        PRINT #4, USING Fmt$; Z(I, J)
      END IF
    NEXT J
  NEXT I
EXIT SUB
END SUB

DEFSNG C, F, I-J, R
SUB PlotFlow (Mbar!, Same$)
DEFINT I-N
SHARED NR%, NC%, OutSum&(), InSum&(), Po&(), Pi&(), D(), E(), R(), M!()
SHARED Expon, Ncoords, X(), Y(), Ib, Xb(), Yb(), Bds()
'Draws arrows to represent movement
Dx = Bds(3) - Bds(1): Dy = Bds(4) - Bds(2)
IF Dy > Dx THEN Dx = Dy
Dxx = Dx / 35
Kk = 0
DIM Xa(2), Ya(2)
FOR I% = 1 TO NR%
  Xa(1) = X(I%): Ya(1) = Y(I%) * 4 / 3
  FOR J% = 1 TO NC%
    IF Same$ = "Y" THEN
      IF (M!(I%, J%) - M!(J%, I%)) > 0 AND M!(I%, J%) > Mbar! THEN GOTO NetFlow
     'Plotting only net flows above the mean
    ELSE
      IF M!(I%, J%) > 0 THEN GOTO NetFlow
     'Plotting only positive fows
    END IF
    GOTO NoFlow
NetFlow:
    IF Same$ = "Y" THEN
      K% = J%
    ELSE
      K% = NR% + J%
    END IF
    Xa(2) = X(K%): Ya(2) = Y(K%) * 4 / 3
    Kk = Kk + 1
   'PRINT #4, " Calling arrow"
   'PRINT #4, Xa(1); Ya(1), Xa(2); Ya(2)
   'PRINT Kk
    CALL Arrow(Xa(), Ya(), Dxx)
NoFlow:
  NEXT J%
NEXT I%
IF Same$ = "Y" THEN
  PRINT Kk; "net movements showing. Press 'enter' to continue"
ELSE
  PRINT Kk; "movements showing. Press `enter' to continue"
END IF
COLOR 0
INPUT "Press enter to continue"; Z$
COLOR 7
CLS
END SUB

DEFSNG I-N
SUB PlusMins (Same$, Bdy$, T$)
'Subroutine to draw + or - symbols on a map"
DEFINT I-N
SHARED NR%, NC%, OutSum&(), InSum&(), Po&(), Pi&(), D(), E(), R(), M!()
SHARED Expon, Ncoords, X(), Y(), Ib, Xb(), Yb(), Bds()
DIM V(Ncoords)
IF Same$ = "Y" THEN
  FOR K = 1 TO Ncoords
    V(K) = OutSum&(K) - InSum&(K)
  NEXT K
ELSE
  J = 0
  FOR K = 1 TO NR%
    V(K) = -OutSum&(K)
  NEXT K
  FOR K = NR% + 1 TO Ncoords
    J = J + 1
    V(K) = InSum&(J)
  NEXT K
END IF

Vmax = -1E+09
CALL Boundry(Bdy$)
Dx = Bds(3) - Bds(1): Dy = Bds(4) - Bds(2)
Range = Dx
IF Dy > Dx THEN Range = Dy
F = 2
Fineness = F ^ 1.5
'Assumes already computed net values
FOR I = 1 TO Ncoords
  IF ABS(V(I)) > Vmax THEN Vmax = ABS(V(I))
NEXT I
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Rescale:
PRINT "   "; T$
Scale = Fineness * Range / (100 * SQR(Vmax))
' Plot symbols
FOR I = 1 TO Ncoords
 'PRINT I, V(I);
  W = Scale * SQR(ABS(V(I))) * SGN(V(I))
 'PRINT W
 'INPUT x$
  Dx = ABS(W) / 2
  U1 = X(I) - Dx
  U2 = X(I) + Dx
  V1 = Y(I) * 4 / 3
  IF W <> 0! THEN
'   Minus sign (in red)
    PSET (U1, V1)
    LINE -(U2, V1), 12
'   Plus sign  (in magenta)
    IF W > 0 THEN
      PSET (U1, V1)
      LINE -(U2, V1), 5
      U1 = X(I)
      V1 = (Y(I) - Dx) * 4 / 3
      V2 = (Y(I) + Dx) * 4 / 3
      PSET (U1, V1)
      LINE -(U1, V2), 5
    END IF
  ELSE
'   Zero value
    PSET (X(I), Y(I) * 4 / 3)
  END IF
NEXT I
CALL Boundry(Bdy$)
COLOR 7
INPUT " Replot, changing scale of symbols?  Y/N (Default=N) ", X$
IF UCASE$(X$) = "Y" THEN
  INPUT " Enter new value for fineness (1 - 5) ", F
  Fineness = F ^ 1.5
  IF F < 1 OR F > 5 THEN GOTO DoneNow
  CLS
  GOTO Rescale
END IF
DoneNow:
ERASE V
END SUB

DEFSNG I-N
'To compute the degree of asymmetry of a SQUARE matrix
'Waldo Tobler 5-1-98
SUB Symmetry (R%, Mbar, M())
DEFINT C, F, I-J
DIM Mp(R%, R%), Mn(R%, R%)
Fmt$ = "###.##"
'Get mean values
FOR I = 1 TO R%
  FOR J = 1 TO R%
    Sum = Sum + M(I, J)
    Mp(I, J) = (M(I, J) + M(J, I)) / 2
    Mn(I, J) = (M(I, J) - M(J, I)) / 2
    'PRINT I; J; M(I, J), Mp(I, J), Mn(I, J)
    IF I = J THEN
      Trace = Trace + M(I, J)
    ELSE
      OffDiag = OffDiag + M(I, J)
    END IF
  NEXT J
NEXT I
SumP = Sum
SumN = 0
PRINT #4, " The estimated array values sum to  "; Sum
PRINT #4, " The trace (diagonal sum) is"; Trace; " or ";
Ts = Trace / Sum * 100
PRINT #4, USING Fmt$; Ts;
PRINT #4, "% of the total"
PRINT #4, " The off-diagonal sum is  "; OffDiag; " or ";
Os = OffDiag / Sum * 100
PRINT #4, USING Fmt$; Os;
PRINT #4, "% of the total"
N% = R% * R%
IF Trace = 0 THEN
  Mbar = Sum / (N% - R%)
  Mpbar = Mbar
  PRINT #4, " The mean of the"; N% - R%; "entries in the array is"; Mbar
ELSE
  Mbar = Sum / N%
  Mpbar = Mbar
  PRINT #4, " The mean of the"; N%; "entries in the array is"; Mbar
END IF
PRINT
'Get variances
FOR I = 1 TO R%
  FOR J = 1 TO R%
    Sm = Sm + (M(I, J) - Mbar) ^ 2
    Sp = Sp + (Mp(I, J) - Mpbar) ^ 2
    Sn = Sn + (Mn(I, J)) ^ 2
  NEXT J
NEXT I
PRINT #4,
PRINT #4, " The estimated array variances:"
PRINT #4, " The total variance is  "; Sm / N%
PRINT #4, " The symmetric variance "; Sp / N%
PRINT #4, " Skew-symmetric variance"; Sn / N%
'Get ratios
PRINT #4,
PRINT #4, " The estimated variance ratios are:"
Symetric = Sp / Sm
Skew = Sn / Sm
PRINT #4, " Symmetric:    "; Symetric
PRINT #4, " Anti-symmtric:"; Skew
PRINT #4, " The percentage asymmetry of the estimated array is";
PRINT #4, USING Fmt$; Skew * 100;
PRINT #4, "%"
END SUB

DEFINT K-N
DEFSNG C, F
FUNCTION TimeDif$ (T1$, T2$)
OneMin = 1 / 60
Hr1 = VAL(MID$(T1$, 1, 2)) + VAL(MID$(T1$, 4, 2)) / 60
Hr2 = VAL(MID$(T2$, 1, 2)) + VAL(MID$(T2$, 4, 2)) / 60
  IF Hr2 - Hr1 < OneMin THEN
  TimeDif$ = "Short"
  ELSE
  TimeDif$ = "Long"
  END IF
END FUNCTION

